
"""
    #####################
    Nmf (``models.nmf``)
    #####################
"""

import nimfa.utils.utils as utils
from nimfa.utils.linalg import *
from nimfa.models import mf_track
from nimfa.methods import seeding


class Nmf(object):
    """
    This class defines a common interface / model to handle NMF models in a generic way.
    
    It contains definitions of the minimum set of generic methods that are used in 
    common computations and matrix factorizations. Besides it contains some quality and performance measures 
    about factorizations. 
    
    .. attribute:: rank
    
        Factorization rank
        
    .. attribute:: V
        
        Target matrix, the matrix for the MF method to estimate. The columns of target matrix V are called samples, the rows of target
        matrix V are called features. Some algorithms (e. g. multiple NMF) specify more than one target matrix. In that case
        target matrices are passed as tuples. Internally, additional attributes with names following Vn pattern are created, 
        where n is the consecutive index of target matrix. Zero index is omitted (there are V, V1, V2, V3, etc. matrices and
        then H, H1, H2, etc. and W, W1, W2, etc. respectively - depends on the algorithm). 
        
    .. attribute:: seed
    
        Method to seed the computation of a factorization
        
    .. attribute:: method
    
        The algorithm to use to perform MF on target matrix
        
    .. attribute:: n_run 
    
        The number of runs of the algorithm
        
    .. attribute:: n_iter
    
        The number of iterations performed
        
    .. attribute:: final_obj
    
        Final value (of the last performed iteration) of the objective function
        
    .. attribute:: callback
    
        A callback function that is called after each run if performing multiple runs 
        
    .. attribute:: options
    
        Runtime / algorithm specific options
        
    .. attribute:: max_iter
    
        Maximum number of factorization iterations
        
    .. attribute:: min_residuals
    
        Minimal required improvement of the residuals from the previous iteration
        
    .. attribute:: test_conv
        
        Indication how often convergence test is done.
    """
    def __init__(self, params):
        """
        Construct generic factorization model.
        
        :param params: MF runtime and algorithm parameters and options. For detailed explanation
           of the general model parameters see :mod:`mf_run`. For algorithm specific
           model options see documentation of chosen factorization method.
        :type params: `dict`
        """
        self.__dict__.update(params)
        # do not copy target and factor matrices into the program
        if sp.isspmatrix(self.V):
            self.V = self.V.tocsr().astype('d')
        else:
            self.V = np.asmatrix(self.V) if self.V.dtype == np.dtype(
                float) else np.asmatrix(self.V, dtype='d')
        if self.V1 is not None:
            if sp.isspmatrix(self.V1):
                self.V1 = self.V1.tocsr().astype('d')
            else:
                self.V1 = np.asmatrix(self.V1) if self.V1.dtype == np.dtype(
                    float) else np.asmatrix(self.V1, dtype='d')
        if self.W is not None:
            if sp.isspmatrix(self.W):
                self.W = self.W.tocsr().astype('d')
            else:
                self.W = np.asmatrix(self.W) if self.W.dtype == np.dtype(
                    float) else np.asmatrix(self.W, dtype='d')
        if self.H is not None:
            if sp.isspmatrix(self.H):
                self.H = self.H.tocsr().astype('d')
            else:
                self.H = np.asmatrix(self.H) if self.H.dtype == np.dtype(
                    float) else np.asmatrix(self.H, dtype='d')
        if self.H1 is not None:
            if sp.isspmatrix(self.H1):
                self.H1 = self.H1.tocsr().astype('d')
            else:
                self.H1 = np.asmatrix(self.H1) if self.H1.dtype == np.dtype(
                    float) else np.asmatrix(self.H1, dtype='d')
        self._compatibility()

    def __call__(self):
        """Run the specified MF algorithm."""
        return self.factorize()

    def basis(self):
        """Return the matrix of basis vectors. See NMF specific model."""

    def target(self, idx=None):
        """Return the target matrix. See NMF specific model."""

    def coef(self, idx=None):
        """
        Return the matrix of mixture coefficients. See NMF specific model.
        
        :param idx: Used in the multiple NMF model. In factorizations following standard
            NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """

    def fitted(self, idx=None):
        """
        Compute the estimated target matrix according to the NMF model. See NMF specific model.

        :param idx: Used in the multiple NMF model. In factorizations following standard
            NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """

    def distance(self, metric='euclidean', idx=None):
        """
        Return the loss function value. See NMF specific model.
        
        :param distance: Specify distance metric to be used. Possible are Euclidean
            and Kullback-Leibler (KL) divergence. Strictly, KL is not a metric.
        :type distance: `str` with values 'euclidean' or 'kl'

        :param idx: Used in the multiple NMF model. In factorizations following
            standard NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """

    def residuals(self, idx=None):
        """
        Compute residuals between the target matrix and its NMF estimate. See NMF specific model.
        
        :param idx: Used in the multiple NMF model. In factorizations following standard
            NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """

    def connectivity(self, H=None, idx=None):
        """
        Compute the connectivity matrix for the samples based on their mixture coefficients. 
        
        The connectivity matrix C is a symmetric matrix which shows the shared membership of the samples: entry C_ij is 1 iff sample i and 
        sample j belong to the same cluster, 0 otherwise. Sample assignment is determined by its largest metagene expression value. 
        
        Return connectivity matrix.
        
        :param idx: Used in the multiple NMF model. In factorizations following
            standard NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        V = self.target(idx)
        H = self.coef(idx) if H is None else H
        _, idx = argmax(H, axis=0)
        mat1 = repmat(idx, V.shape[1], 1)
        mat2 = repmat(idx.T, 1, V.shape[1])
        conn = elop(mat1, mat2, eq)
        if sp.isspmatrix(conn):
            return conn.__class__(conn, dtype='d')
        else:
            return np.mat(conn, dtype='d')

    def consensus(self, idx=None):
        """
        Compute consensus matrix as the mean connectivity matrix across multiple runs of the factorization. It has been
        proposed by [Brunet2004]_ to help visualize and measure the stability of the clusters obtained by NMF.
        
        Tracking of matrix factors across multiple runs must be enabled for computing consensus matrix. For results
        of a single NMF run, the consensus matrix reduces to the connectivity matrix.
        
        :param idx: Used in the multiple NMF model. In factorizations following
            standard NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        V = self.target(idx)
        if self.track_factor:
            if sp.isspmatrix(V):
                cons = V.__class__((V.shape[1], V.shape[1]), dtype=V.dtype)
            else:
                cons = np.mat(np.zeros((V.shape[1], V.shape[1])))
            for i in range(self.n_run):
                cons += self.connectivity(
                    H=self.tracker.get_factor(i).H, idx=idx)
            return sop(cons, self.n_run, div)
        else:
            return self.connectivity(H=self.coef(idx), idx=idx)

    def dim(self, idx=None):
        """
        Return triple containing the dimension of the target matrix and matrix factorization rank.
        
        :param idx: Used in the multiple NMF model. In factorizations following
            standard NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        V = self.target(idx)
        return (V.shape[0], V.shape[1], self.rank)

    def entropy(self, membership=None, idx=None):
        """
        Compute the entropy of the NMF model given a priori known groups of
        samples [Park2007]_.
        
        The entropy is a measure of performance of a clustering method in
        recovering classes defined by a list a priori known (true class labels).
        
        Return the real number. The smaller the entropy, the better the
        clustering performance.
        
        :param membership: Specify known class membership for each sample. 
        :type membership: `list`

        :param idx: Used in the multiple NMF model. In factorizations following
           standard NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        V = self.target(idx)
        if not membership:
            raise utils.MFError(
                "Known class membership for each sample is not specified.")
        n = V.shape[1]
        mbs = np.array(self.predict(what="samples", prob=False, idx=idx)).squeeze()
        dmbs, dmembership = {}, {}
        [dmbs.setdefault(mbs[i], set()).add(i) for i in range(len(mbs))]
        [dmembership.setdefault(membership[i], set()).add(i) for i in range(len(membership))]
        entropy = 0.
        for k in dmbs:
            for j in dmembership:
                entropy += len(dmbs[k].intersection(dmembership[j])) * np.log2(len(dmbs[k].intersection(dmembership[j])) / float(len(dmbs[k])))
        entropy *=  -1. / (n * np.log2(len(dmembership)))
        return entropy

    def predict(self, what='samples', prob=False, idx=None):
        """
        Compute the dominant basis components. The dominant basis component is
        computed as the row index for which the entry is the maximum within the column.
        
        If ``prob`` is not specified, list is returned which contains computed index
        for each sample (feature). Otherwise tuple is returned where first element
        is a list as specified before and second element is a list of associated
        probabilities, relative contribution of the maximum entry within each column. 
        
        :param what: Specify target for dominant basis components computation. Two values
           are possible, 'samples' or 'features'. When what='samples' is specified,
           dominant basis component for each sample is determined based on its associated
           entries in the mixture coefficient matrix (H). When what='features' computation
           is performed on the transposed basis matrix (W.T).
        :type what: `str`

        :param prob: Specify dominant basis components probability inclusion.
        :type prob: `bool` equivalent

        :param idx: Used in the multiple NMF model. In factorizations following
           standard NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        X = self.coef(idx) if what == "samples" else self.basis(
        ).T if what == "features" else None
        if X is None:
            raise utils.MFError(
                "Dominant basis components can be computed for samples or features.")
        eX, idxX = argmax(X, axis=0)
        if not prob:
            return idxX
        sums = X.sum(axis=0)
        prob = [e / (sums[0, s] + 1e-5) for e, s in zip(eX, list(range(X.shape[1])))]
        return idxX, prob

    def evar(self, idx=None):
        """
        Compute the explained variance of the NMF estimate of the target matrix.
        
        This measure can be used for comparing the ability of models for accurately
        reproducing the original target matrix. Some methods specifically aim at
        minimizing the RSS and maximizing the explained variance while others not, which
        one should note when using this measure. 
        
        :param idx: Used in the multiple NMF model. In factorizations following
           standard NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        V = self.target(idx)
        return 1. - self.rss(idx=idx) / multiply(V, V).sum()

    def score_features(self, idx=None):
        """
        Score features in terms of their specificity to the basis vectors [Park2007]_.
        
        A row vector of the basis matrix (W) indicates contributions of a feature
        to the r (i.e. columns of W) latent components. It might be informative to
        investigate features that have strong component-specific membership values
        to the latent components.
        
        Return array with feature scores. Feature scores are real-valued from interval [0,1].
        Higher value indicates greater feature specificity.

        :param idx: Used in the multiple NMF model. In standard NMF model or nonsmooth NMF model
           ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        W = self.basis()
        scores = np.zeros(W.shape[0])
        for f in range(W.shape[0]):
            # probability that the i-th feature contributes to q-th basis vector.
            prob = W[f, :] / (W[f, :].sum() + np.finfo(W.dtype).eps)
            prob = prob.todense() if sp.isspmatrix(prob) else prob
            scores[f] = np.dot(prob, np.log2(prob + np.finfo(W.dtype).eps).T)
        scores = 1. + 1. / np.log2(W.shape[1]) * scores
        return scores

    def select_features(self, idx=None):
        """
        Compute the most basis-specific features for each basis vector [Park2007]_.
        
        [Park2007]_ scoring schema and feature selection method is used. The features are
        first scored using the :func:`score_features`. Then only the features that fulfill
        both the following criteria are retained:

        #. score greater than u + 3s, where u and s are the median and the median
           absolute deviation (MAD) of the scores, resp.,
        #. the maximum contribution to a basis component (i.e the maximal value in
           the corresponding row of the basis matrix (W)) is larger
           than the median of all contributions (i.e. of all elements of basis matrix (W)).
        
        Return a boolean array indicating whether features were selected.
        
        :param idx: Used in the multiple NMF model. In standard NMF model or nonsmooth NMF
           model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        scores = self.score_features(idx=idx)
        th = np.median(scores) + 3 * np.median(abs(scores - np.median(scores)))
        sel = scores > th
        W = self.basis()
        m = np.median(W.toarray() if sp.isspmatrix(W) else W.tolist())
        sel = np.array([sel[i] and np.max(W[i, :].toarray() if sp.isspmatrix(W) else W[i,:]) > m
               for i in range(W.shape[0])])
        return sel

    def purity(self, membership=None, idx=None):
        """
        Compute the purity given a priori known groups of samples [Park2007]_.
        
        The purity is a measure of performance of a clustering method in recovering
        classes defined by a list a priori known (true class labels).
        
        Return the real number in [0,1]. The larger the purity, the better the
        clustering performance.
        
        :param membership: Specify known class membership for each sample. 
        :type membership: `list`

        :param idx: Used in the multiple NMF model. In factorizations following
           standard NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        V = self.target(idx)
        if not membership:
            raise utils.MFError(
                "Known class membership for each sample is not specified.")
        n = V.shape[1]
        mbs = np.array(self.predict(what="samples", prob=False, idx=idx)).squeeze()
        dmbs, dmembership = {}, {}
        [dmbs.setdefault(mbs[i], set()).add(i) for i in range(len(mbs))]
        [dmembership.setdefault(membership[i], set()).add(i) for i in range(len(membership))]
        return 1. / n * sum(np.max([len(dmbs[k].intersection(dmembership[j])) for j in dmembership]) for k in dmbs)

    def rss(self, idx=None):
        """
        Compute Residual Sum of Squares (RSS) between NMF estimate and
        target matrix [Hutchins2008]_.
        
        This measure can be used to estimate optimal factorization rank.
        [Hutchins2008]_ suggested to choose the first value where the RSS curve
        presents an inflection point. [Frigyesi2008]_ suggested to use the
        smallest value at which the decrease in the RSS is lower than the
        decrease of the RSS obtained from random data.
        
        RSS tells us how much of the variation in the dependent variables our
        model did not explain.
        
        Return real value.
        
        :param idx: Used in the multiple NMF model. In factorizations following
           standard NMF model or nonsmooth NMF model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        X = self.residuals(idx=idx)
        return multiply(X, X).sum()

    def sparseness(self, idx=None):
        """
        Compute sparseness of matrix (basis vectors matrix, mixture coefficients) [Hoyer2004]_.

        Sparseness of a vector quantifies how much energy is packed into its components.
        The sparseness of a vector is a real number in [0, 1], where sparser vector
        has value closer to 1. Sparseness is 1 iff the vector contains a single
        nonzero component and is equal to 0 iff all components of the vector are equal.
        
        Sparseness of a matrix is mean sparseness of its column vectors.
        
        Return tuple that contains sparseness of the basis and mixture coefficients matrices. 
        
        :param idx: Used in the multiple NMF model. In standard NMF model or nonsmooth NMF
           model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        def sparseness(x):
            eps = np.finfo(x.dtype).eps if 'int' not in str(x.dtype) else 1e-9
            x1 = sqrt(x.shape[0]) - (abs(x).sum() + eps) / \
                (sqrt(multiply(x, x).sum()) + eps)
            x2 = sqrt(x.shape[0]) - 1
            return x1 / x2
        W = self.basis()
        H = self.coef(idx)
        spars_W = np.mean([sparseness(W[:, i]) for i in range(W.shape[1])])
        spars_H = np.mean([sparseness(H[:, i]) for i in range(H.shape[1])])
        return spars_W, spars_H

    def coph_cor(self, idx=None):
        """
        Compute cophenetic correlation coefficient of consensus matrix, generally obtained from multiple NMF runs. 
        
        The cophenetic correlation coefficient is measure which indicates the dispersion of the consensus matrix and is based 
        on the average of connectivity matrices. It measures the stability of the clusters obtained from NMF. 
        It is computed as the Pearson correlation of two distance matrices: the first is the distance between samples induced by the 
        consensus matrix; the second is the distance between samples induced by the linkage used in the reordering of the consensus 
        matrix [Brunet2004]_.
        
        Return real number. In a perfect consensus matrix, cophenetic correlation equals 1. When the entries in consensus matrix are
        scattered between 0 and 1, the cophenetic correlation is < 1. We observe how this coefficient changes as factorization rank 
        increases. We select the first rank, where the magnitude of the cophenetic correlation coefficient begins to fall [Brunet2004]_.
        
        :param idx: Used in the multiple NMF model. In factorizations following standard NMF model or nonsmooth NMF model
                    :param:`idx` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        A = self.consensus(idx=idx)
        # upper diagonal elements of consensus
        avec = np.array([A[i, j] for i in range(A.shape[0] - 1)
                        for j in range(i + 1, A.shape[1])])
        # consensus entries are similarities, conversion to distances
        Y = 1 - avec
        Z = linkage(Y, method='average')
        # cophenetic correlation coefficient of a hierarchical clustering
        # defined by the linkage matrix Z and matrix Y from which Z was
        # generated
        return cophenet(Z, Y)[0]

    def dispersion(self, idx=None):
        """
        Compute dispersion coefficient of consensus matrix
        
        Dispersion coefficient [Park2007]_ measures the reproducibility of clusters obtained
        from multiple NMF runs.
        
        Return the real value in [0,1]. Dispersion is 1 for a perfect consensus matrix and
        has value in [0,0] for a scattered consensus matrix.

        :param idx: Used in the multiple NMF model. In standard NMF model or nonsmooth NMF
           model ``idx`` is always None.
        :type idx: None or `str` with values 'coef' or 'coef1' (`int` value of 0 or 1, respectively) 
        """
        C = self.consensus(idx=idx)
        dispersion = np.sum(4 * np.multiply(C - 0.5, C - 0.5)) / C.size
        return dispersion

    def estimate_rank(self, rank_range=range(30, 51), n_run=10, idx=0, what='all'):
        """
        Choosing factorization parameters carefully is vital for success of a factorization.
        However, the most critical parameter is factorization rank. This method tries
        different values for ranks, performs factorizations, computes some quality
        measures of the results and chooses the best value according to [Brunet2004]_
        and [Hutchins2008]_.
        
        .. note:: The process of rank estimation can be lengthy.   
        
        .. note:: Matrix factors are tracked during rank estimation. This is needed
           for computing cophenetic correlation coefficient.
        
        Return a `dict` (keys are values of rank from range, values are `dict`s of measures)
        of quality measures for each value in rank's range. This can be passed to the
        visualization model, from which estimated rank can be established.
        
        :param rank_range: Range of factorization ranks to try. Default is ``range(30, 51)``.
        :type rank_range: list or tuple like range of `int`

        :param n_run: The number of runs to be performed for each value in range. Default is 10.  
        :type n_run: `int`

        :param what: Specify quality measures of the results computed for each rank.
           By default, summary of the fitted factorization model is computed. Instead,
           user can supply list of strings that matches some of the following quality measures:
                     
             * `sparseness`
             * `rss`
             * `evar`
             * `residuals`
             * `connectivity`
             * `dispersion`
             * `cophenetic`
             * `consensus`
             * `euclidean`
             * `kl`
        :type what: list or tuple like of `str`

        :param idx: Name of the matrix (coefficient) matrix. Used only in the multiple
           NMF model. Default is 0 (first coefficient matrix).
        :type idx: `str` or `int`
        """
        self.n_run = n_run
        self.track_factor = True
        self.tracker = mf_track.Mf_track()

        def _measures(measure):
            return {
                'sparseness': fctr.fit.sparseness,
                'rss': fctr.fit.rss,
                'evar': fctr.fit.evar,
                'residuals': fctr.fit.residuals,
                'connectivity': fctr.fit.connectivity,
                'dispersion': fctr.fit.dispersion,
                'cophenetic': fctr.fit.coph_cor,
                'consensus': fctr.fit.consensus}[measure]
        summaries = {}
        for rank in rank_range:
            self.rank = rank
            fctr = self()
            if what == 'all':
                summaries[rank] = fctr.summary(idx)
            else:
                summaries[rank] = {
                    'rank': fctr.fit.rank,
                    'n_iter': fctr.fit.n_iter,
                    'n_run': fctr.fit.n_run}
                for measure in what:
                    if measure == 'euclidean':
                        summaries[rank][measure] = fctr.distance(
                            metric='euclidean', idx=idx)
                    elif measure == 'kl':
                        summaries[rank][measure] = fctr.distance(
                            metric='kl', idx=idx)
                    else:
                        summaries[rank][measure] = _measures(
                            measure)(idx=idx)
        return summaries

    def _compatibility(self):
        """
        Check if chosen seeding method is compatible with chosen factorization
        method or fixed initialization is passed.

        :param mf_model: The underlying initialized model of matrix factorization.
        :type mf_model: Class inheriting :class:`models.nmf.Nmf`
        """
        W = self.basis()
        H = self.coef(0)
        H1 = self.coef(1) if self.model_name == 'mm' else None
        if self.seed is None and W is None and H is None and H1 is None:
            self.seed = None if "none" in self.aseeds else "random"
        if W is not None and H is not None:
            if self.seed is not None and self.seed is not "fixed":
                raise utils.MFError("Initial factorization is fixed.")
            else:
                self.seed = seeding.fixed.Fixed()
                self.seed._set_fixed(W=W, H=H, H1=H1)
        self.__is_smdefined()
        self.__compatibility()

    def __is_smdefined(self):
        """Check if MF and seeding methods are well defined."""
        if isinstance(self.seed, str):
            if self.seed in seeding.methods:
                self.seed = seeding.methods[self.seed]()
            else:
                raise utils.MFError("Unrecognized seeding method.")
        else:
            if not str(self.seed).lower() in seeding.methods:
                raise utils.MFError("Unrecognized seeding method.")

    def __compatibility(self):
        """Check if MF model is compatible with the seeding method."""
        if not str(self.seed).lower() in self.aseeds:
            raise utils.MFError("MF model is incompatible with the seeding method.")



import numpy as np
import scipy.sparse as spr

import nimfa

V = spr.csr_matrix([[1, 0, 2, 4], [0, 0, 6, 3], [4, 0, 5, 6]])
print('Target:\n%s' % V.todense())

nmf = nimfa.Nmf(V, max_iter=200, rank=2, update='euclidean', objective='fro')
nmf_fit = nmf()

W = nmf_fit.basis()
print('Basis matrix:\n%s' % W.todense())

H = nmf_fit.coef()
print('Mixture matrix:\n%s' % H.todense())

print('Euclidean distance: %5.3f' % nmf_fit.distance(metric='euclidean'))

sm = nmf_fit.summary()
print('Sparseness Basis: %5.3f  Mixture: %5.3f' % (sm['sparseness'][0], sm['sparseness'][1]))
print('Iterations: %d' % sm['n_iter'])
print('Target estimate:\n%s' % np.dot(W.todense(), H.todense()))




"""
#########################################
Snmnmf (``methods.factorization.snmnmf``)
#########################################

**Sparse Network-Regularized Multiple Nonnegative Matrix Factorization
(SNMNMF)** [Zhang2011]_.

It is semi-supervised learning method with constraints (e. g. in comodule
identification, any variables linked in A or B, are more likely placed in the
same comodule) to improve relevance and narrow down the search space.

The advantage of this method is the integration of multiple matrices for
multiple types of variables (standard NMF methods can be applied to a target
matrix containing just one type of variable) together with prior knowledge
(e. g. network representing relationship among variables). 

The objective function in [Zhang2011]_ has three components:
    #. first component models miRNA and gene expression profiles;
    #. second component models gene-gene network interactions;
    #. third component models predicted miRNA-gene interactions.
 
The inputs for the SNMNMF are:
    #. two sets of expression profiles (represented by the matrices V and V1 of
       shape s x m, s x n, respectively) for miRNA and genes measured on the same
       set of samples;
    #. (PRIOR KNOWLEDGE) a gene-gene interaction network (represented by the
       matrix A of shape n x n), including protein-protein interactions and
       DNA-protein interactions; the network is presented in the form of the
       adjacency matrix of gene network;
    #. (PRIOR KNOWLEDGE) a list of predicted miRNA-gene regulatory interactions
       (represented by the matrix B of shape m x n) based on sequence data; the
       network is presented in the form of the adjacency matrix of a bipartite
       miRNA-gene network. Network regularized constraints are used to enforce
       "must-link" constraints and to ensure that genes with known interactions
       have similar coefficient profiles.
       
Gene and miRNA expression matrices are simultaneously factored into a common
basis matrix (W) and two coefficients matrices (H and H1). Additional knowledge
is incorporated into this framework with network regularized constraints.
Because of the imposed sparsity constraints easily interpretable solution is
obtained. In [Zhang2011]_ decomposed matrix components are used to provide
information about miRNA-gene regulatory comodules. They identified the comodules
based on shared components (a column in basis matrix W) with significant
association values in the corresponding rows of coefficients matrices, H1 and H2.

In SNMNMF a strategy suggested by Kim and Park (2007) is adopted to make the
coefficient matrices sparse.

.. note:: In [Zhang2011]_ ``H1`` and ``H2`` notation corresponds to the ``H``
and ``H1`` here, respectively.

.. literalinclude:: /code/snippet_snmnmf.py

"""

from nimfa.models import *
from nimfa.utils import *
from nimfa.utils.linalg import *

__all__ = ['Snmnmf']


class Snmnmf(nmf_mm.Nmf_mm):
    """
    :param V: The target matrix to estimate.
    :type V: Instance of the :class:`scipy.sparse` sparse matrices types,
       :class:`numpy.ndarray`, :class:`numpy.matrix` or tuple of instances of
       the latter classes.

    :param V1: The target matrix to estimate. Used by algorithms that consider
       more than one target matrix.
    :type V1: Instance of the :class:`scipy.sparse` sparse matrices types,
       :class:`numpy.ndarray`, :class:`numpy.matrix` or tuple of instances of
       the latter classes.

    :param seed: Specify method to seed the computation of a factorization. If
       specified :param:`W` and :param:`H` seeding must be None. If neither seeding
       method or initial fixed factorization is specified, random initialization is
       used.
    :type seed: `str` naming the method or :class:`methods.seeding.nndsvd.Nndsvd`
       or None

    :param W: Specify initial factorization of basis matrix W. Default is None.
       When specified, :param:`seed` must be None.
    :type W: :class:`scipy.sparse` or :class:`numpy.ndarray` or
       :class:`numpy.matrix` or None

    :param H: Specify initial factorization of mixture matrix H. Default is None.
       When specified, :param:`seed` must be None.
    :type H: Instance of the :class:`scipy.sparse` sparse matrices types,
       :class:`numpy.ndarray`, :class:`numpy.matrix`, tuple of instances of the
       latter classes or None

    :param rank: The factorization rank to achieve. Default is 30.
    :type rank: `int`

    :param n_run: It specifies the number of runs of the algorithm. Default is
       1. If multiple runs are performed, fitted factorization model with the
       lowest objective function value is retained.
    :type n_run: `int`

    :param callback: Pass a callback function that is called after each run when
       performing multiple runs. This is useful if one wants to save summary
       measures or process the result before it gets discarded. The callback
       function is called with only one argument :class:`models.mf_fit.Mf_fit` that
       contains the fitted model. Default is None.
    :type callback: `function`

    :param callback_init: Pass a callback function that is called after each
       initialization of the matrix factors. In case of multiple runs the function
       is called before each run (more precisely after initialization and before
       the factorization of each run). In case of single run, the passed callback
       function is called after the only initialization of the matrix factors.
       This is useful if one wants to obtain the initialized matrix factors for
       further analysis or additional info about initialized factorization model.
       The callback function is called with only one argument
       :class:`models.mf_fit.Mf_fit` that (among others) contains also initialized
       matrix factors. Default is None.
    :type callback_init: `function`

    :param track_factor: When :param:`track_factor` is specified, the fitted
        factorization model is tracked during multiple runs of the algorithm. This
        option is taken into account only when multiple runs are executed
        (:param:`n_run` > 1). From each run of the factorization all matrix factors
        are retained, which can be very space consuming. If space is the problem
        setting the callback function with :param:`callback` is advised which is
        executed after each run. Tracking is useful for performing some quality or
        performance measures (e.g. cophenetic correlation, consensus matrix,
        dispersion). By default fitted model is not tracked.
    :type track_factor: `bool`

    :param track_error: Tracking the residuals error. Only the residuals from
        each iteration of the factorization are retained. Error tracking is not
        space consuming. By default residuals are not tracked and only the final
        residuals are saved. It can be used for plotting the trajectory of the
        residuals.
    :type track_error: `bool`

    :param A: Adjacency matrix of gene-gene interaction network (dimension:
       V1.shape[1] x V1.shape[1]).
    :type A: :class:`scipy.sparse` of format csr, csc, coo, bsr, dok, lil, dia
       or :class:`numpy.matrix`

    :param B: Adjacency matrix of a bipartite miRNA-gene network, predicted
       miRNA-target interactions (dimension: V.shape[1] x V1.shape[1]).
    :type B: :class:`scipy.sparse` of format csr, csc, coo, bsr, dok, lil, dia or
       :class:`numpy.matrix`

    :param gamma: Limit the growth of the basis matrix (W). Default is 0.01.
    :type gamma: `float`

    :param gamma_1: Encourage sparsity of the mixture (coefficient) matrices (H
       and H1). Default is 0.01.
    :type gamma_1: `float`

    :param lamb: Weight for the must-link constraints defined in ``A``.
       Default is 0.01.
    :type lamb: `float`

    :param lamb_1: Weight for the must-link constraints define in ``B``.
       Default is 0.01.
    :type lamb_1: `float`

    **Stopping criterion**

    Factorization terminates if any of specified criteria is satisfied.

    :param max_iter: Maximum number of factorization iterations. Note that the
       number of iterations depends on the speed of method convergence. Default
       is 30.
    :type max_iter: `int`

    :param min_residuals: Minimal required improvement of the residuals from the
       previous iteration. They are computed between the target matrix and its MF
       estimate using the objective function associated to the MF algorithm.
       Default is None.
    :type min_residuals: `float`

    :param test_conv: It indicates how often convergence test is done. By
       default convergence is tested each iteration.
    :type test_conv: `int`
    """
    def __init__(self, V, V1, seed=None, W=None, H=None, H1=None,
                 rank=30, max_iter=30, min_residuals=1e-5, test_conv=None,
                 n_run=1, callback=None, callback_init=None, track_factor=False,
                 track_error=False, A=None, B=None, gamma=0.01, gamma_1=0.01,
                 lamb=0.01, lamb_1=0.01, **options):
        self.name = "snmnmf"
        self.aseeds = ["random", "fixed", "nndsvd", "random_c", "random_vcol"]
        nmf_mm.Nmf_mm.__init__(self, vars())
        if self.A is None:
            self.A = sp.csr_matrix((self.V1.shape[1], self.V1.shape[1]))
        self.A = self.A.tocsr() if sp.isspmatrix(self.A) else np.mat(self.A)
        if self.B is None:
            self.B = sp.csr_matrix((self.V.shape[1], self.V1.shape[1]))
        self.B = self.B.tocsr() if sp.isspmatrix(self.B) else np.mat(self.B)
        self.tracker = mf_track.Mf_track() if self.track_factor and self.n_run > 1 \
                                              or self.track_error else None

    def factorize(self):
        """
        Compute matrix factorization.
         
        Return fitted factorization model.
        """
        if self.V.shape[0] != self.V1.shape[0]:
            raise utils.MFError(
                "Input matrices should have the same number of rows.")

        for run in range(self.n_run):
            self.options.update({'idx': 0})
            self.W, self.H = self.seed.initialize(
                self.V, self.rank, self.options)
            self.options.update({'idx': 1})
            _, self.H1 = self.seed.initialize(self.V1, self.rank, self.options)
            self.options.pop('idx')
            p_obj = c_obj = sys.float_info.max
            best_obj = c_obj if run == 0 else best_obj
            self.err_avg = 1
            iter = 0
            if self.callback_init:
                self.final_obj = c_obj
                self.n_iter = iter
                mffit = mf_fit.Mf_fit(self)
                self.callback_init(mffit)
            while self.is_satisfied(p_obj, c_obj, iter):
                p_obj = c_obj if not self.test_conv or iter % self.test_conv == 0 else p_obj
                self.update(iter)
                iter += 1
                c_obj = self.objective(
                ) if not self.test_conv or iter % self.test_conv == 0 else c_obj
                if self.track_error:
                    self.tracker.track_error(run, c_obj)
            if self.callback:
                self.final_obj = c_obj
                self.n_iter = iter
                mffit = mf_fit.Mf_fit(self)
                self.callback(mffit)
            if self.track_factor:
                self.tracker.track_factor(
                    run, W=self.W, H=self.H, H1=self.H1.copy(),
                    final_obj=c_obj, n_iter=iter)
            # if multiple runs are performed, fitted factorization model with
            # the lowest objective function value is retained
            if c_obj <= best_obj or run == 0:
                best_obj = c_obj
                self.n_iter = iter
                self.final_obj = c_obj
                mffit = mf_fit.Mf_fit(copy.deepcopy(self))

        mffit.fit.tracker = self.tracker
        return mffit

    def is_satisfied(self, p_obj, c_obj, iter):
        """
        Compute the satisfiability of the stopping criteria based on stopping
        parameters and objective function value.
        
        Return logical value denoting factorization continuation. 
        
        :param p_obj: Objective function value from previous iteration. 
        :type p_obj: `float`
        :param c_obj: Current objective function value.
        :type c_obj: `float`
        :param iter: Current iteration number. 
        :type iter: `int`
        """
        if self.err_avg < 1e-5:
            return False
        if self.max_iter and self.max_iter <= iter:
            return False
        if self.test_conv and iter % self.test_conv != 0:
            return True
        if self.min_residuals and iter > 0 and p_obj - c_obj < self.min_residuals:
            return False
        if iter > 0 and c_obj > p_obj:
            return False
        return True

    def update(self, iter):
        """Update basis and mixture matrix."""
        # update basis matrix
        temp_w1 = dot(self.V, self.H.T) + dot(self.V1, self.H1.T)
        temp_w2 = dot(self.W, dot(self.H, self.H.T) + dot(
            self.H1, self.H1.T)) + self.gamma / 2. * self.W
        self.W = multiply(self.W, elop(temp_w1, temp_w2, div))
        # update mixture matrices
        # update H1
        temp = sop(dot(self.W.T, self.W), s=self.gamma_1, op=add)
        temp_h1 = dot(self.W.T, self.V) + \
            self.lamb_1 / 2. * dot(self.H1, self.B.T)
        HH1 = multiply(self.H, elop(temp_h1, dot(temp, self.H), div))
        temp_h3 = dot(self.W.T, self.V1) + self.lamb * dot(
            self.H1, self.A) + self.lamb_1 / 2. * dot(self.H, self.B)
        temp_h4 = dot(temp, self.H1)
        self.H1 = multiply(self.H1, elop(temp_h3, temp_h4, div))
        # update H
        self.H = HH1

    def objective(self):
        """Compute three component objective function as defined in [Zhang2011]_."""
        err_avg1 = abs(self.V - dot(self.W, self.H)).mean() / self.V.mean()
        err_avg2 = abs(self.V1 - dot(self.W, self.H1)).mean() / self.V1.mean()
        self.err_avg = err_avg1 + err_avg2
        R1 = self.V - dot(self.W, self.H)
        eucl1 = (multiply(R1, R1)).sum()
        R2 = self.V1 - dot(self.W, self.H1)
        eucl2 = (multiply(R2, R2)).sum()
        tr1 = trace(dot(dot(self.H1, self.A), self.H1.T))
        tr2 = trace(dot(dot(self.H, self.B), self.H1.T))
        s1 = multiply(self.W, self.W).sum()
        s2 = multiply(self.H, self.H).sum() + multiply(self.H1, self.H1).sum()
        reg = - self.lamb * tr1 - self.lamb_1 * tr2 + self.gamma * s1 + self.gamma_1 * s2
        return eucl1 + eucl2  + reg

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name




"""
#################
Nimfa (``nimfa``)
#################

Nimfa is a Python module which includes a number of published matrix
factorization algorithms, initialization methods, quality and performance
measures and facilitates the combination of these to produce new strategies.
The library represents a unified and efficient interface to matrix
factorization algorithms and methods.
"""

__license__ = 'BSD'
__version__ = '1.3.5'
__maintainer__ = 'Marinka Zitnik'
__email__ = 'marinka@cs.stanford.edu'


from nimfa import examples
from nimfa.methods.factorization import *
from .version import \
    short_version as __version__, git_revision as __git_version__




"""
    #######################################
    ALL AML Leukemia (``examples.aml_all``)
    #######################################
    
    This module demonstrates the ability of NMF to recover meaningful biological information from 
    cancer related microarray data. NMF appears to have advantages over other methods such as HC or SOM. 
    Instead of separating gene clusters based on distance computation, NMF detects context-dependent patterns 
    of gene expression in complex biological systems.
    
    `Leukemia`_ data set is used in this example. This data set is a benchmark in the cancer classification
    community. It contains two ALL samples that are consistently misclassified or classified with low 
    confidence by most methods. There are a number of possible explanations for this, 
    including incorrect diagnosis of the samples. They are included them in example.The distinction between AML 
    and ALL, as well as the division of ALL into T and B cell subtypes is well known. 
    
    .. note:: Leukemia data set used in this example is included in the `datasets` and does not need to be
              downloaded. However, download links are listed in the ``datasets``. To run the example, the data set
              must exist in the ``ALL_AML`` directory under `data sets`. 
    
    .. _Leukemia: http://orange.biolab.si/data sets/leukemia.htm 
    
    This example is inspired by [Brunet2004]_. In [Brunet2004]_ authors applied NMF to the leukemia data set. With rank, rank = 2, 
    NMF recovered the AML-ALL biological distinction with high accuracy and robustness. Higher ranks revealed further
    partitioning of the samples. Clear block diagonal patterns in reordered consensus matrices attest to the 
    robustness of models with 2, 3 and 4 classes. 
    
    .. figure:: /images/all_aml_consensus2.png
       :scale: 60 %
       :alt: Consensus matrix generated for rank, rank = 2. 
       :align: center

       Reordered consensus matrix generated for rank, rank = 2. Reordered consensus matrix averages 50 connectivity 
       matrices computed at rank = 2, 3 for the leukemia data set with the 5000 most highly varying genes 
       according to their coefficient of variation. Samples are hierarchically clustered by using 
       distances derived from consensus clustering matrix entries, coloured from 0 (deep blue, samples
       are never in the same cluster) to 1 (dark red, samples are always in the same cluster).   
       
       
    .. figure:: /images/all_aml_consensus3.png
       :scale: 60 %
       :alt: Consensus matrix generated for rank, rank = 3.
       :align: center 

       Reordered consensus matrix generated for rank, rank = 3.
    
    
    .. table:: Standard NMF Class assignments obtained with this example for rank = 2 and rank = 3. 

       ====================  ========== ==========
              Sample          rank = 2   rank = 3
       ====================  ========== ==========
        ALL_19769_B-cell        0            2
        ALL_23953_B-cell        0            2
        ALL_28373_B-cell        0            2
        ALL_9335_B-cell         0            2
        ALL_9692_B-cell         0            2
        ALL_14749_B-cell        0            2
        ALL_17281_B-cell        0            2
        ALL_19183_B-cell        0            2
        ALL_20414_B-cell        0            2
        ALL_21302_B-cell        0            1
        ALL_549_B-cell          0            2
        ALL_17929_B-cell        0            2
        ALL_20185_B-cell        0            2
        ALL_11103_B-cell        0            2
        ALL_18239_B-cell        0            2
        ALL_5982_B-cell         0            2
        ALL_7092_B-cell         0            2
        ALL_R11_B-cell          0            2
        ALL_R23_B-cell          0            2
        ALL_16415_T-cell        0            1
        ALL_19881_T-cell        0            1
        ALL_9186_T-cell         0            1
        ALL_9723_T-cell         0            1
        ALL_17269_T-cell        0            1
        ALL_14402_T-cell        0            1
        ALL_17638_T-cell        0            1
        ALL_22474_T-cell        0            1       
        AML_12                  1            0
        AML_13                  0            0
        AML_14                  1            1
        AML_16                  1            0
        AML_20                  1            0
        AML_1                   1            0
        AML_2                   1            0
        AML_3                   1            0
        AML_5                   1            0 
        AML_6                   1            0
        AML_7                   1            0
       ====================  ========== ========== 
    
    To run the example simply type::
        
        python all_aml.py
        
    or call the module's function::
    
        import nimfa.examples
        nimfa.examples.all_aml.run()
        
    .. note:: This example uses ``matplotlib`` library for producing a heatmap of a consensus matrix.
"""

from os.path import dirname, abspath, join
from warnings import warn

from scipy.cluster.hierarchy import linkage, leaves_list
from scipy.spatial.distance import squareform
import numpy as np

import nimfa

try:
    from matplotlib.pyplot import savefig, imshow, set_cmap
except ImportError as exc:
    warn("Matplotlib must be installed to run ALL AML example.")


def run():
    """Run Standard NMF on leukemia data set. """
    V = read()
    for rank in range(2, 4):
        run_one(V, rank)


def run_one(V, rank):
    """
    Run standard NMF on leukemia data set. 50 runs of Standard NMF are performed and obtained consensus matrix
    averages all 50 connectivity matrices.

    :param V: Target matrix with gene expression data.
    :type V: `numpy.ndarray`
    :param rank: Factorization rank.
    :type rank: `int`
    """
    print("================= Rank = %d =================" % rank)
    consensus = np.zeros((V.shape[1], V.shape[1]))
    for i in range(50):
        nmf = nimfa.Nmf(V, rank=rank, seed="random_vcol", max_iter=200, update='euclidean',
                        objective='conn', conn_change=40)
        fit = nmf()
        print("%2d/50 : %s - init: %s (%3d/200 iterations)" % (i + 1, fit.fit,
                                                               fit.fit.seed, fit.fit.n_iter))
        consensus += fit.fit.connectivity()
    consensus /= 50.
    p_consensus = reorder(consensus)
    plot(p_consensus, rank)


def plot(C, rank):
    """
    Plot reordered consensus matrix.

    :param C: Reordered consensus matrix.
    :type C: numpy.ndarray`
    :param rank: Factorization rank.
    :type rank: `int`
    """
    imshow(C)
    set_cmap("RdBu_r")
    savefig("all_aml_consensus_%d.png" % rank)


def reorder(C):
    """
    Reorder consensus matrix.

    :param C: Consensus matrix.
    :type C: `numpy.ndarray`
    """
    Y = 1 - C
    Z = linkage(squareform(Y), method='average')
    ivl = leaves_list(Z)
    ivl = ivl[::-1]
    return C[:, ivl][ivl, :]


def read():
    """
    Read ALL AML gene expression data. The matrix's shape is 5000 (genes) x 38 (samples).
    It contains only positive data.

    Return the gene expression data matrix.
    """
    fname = join(dirname(dirname(abspath(__file__))), 'datasets', 'ALL_AML', 'ALL_AML_data.txt')
    V = np.loadtxt(fname)
    return V


if __name__ == "__main__":
    """Run the ALL AML example."""
    run()




"""
    ##################################
    Documents (``examples.documents``)
    ##################################

    In this example of text analysis we consider the text processing application inspired by [Albright2006]_.
    
    We used the Medlars data set, which is a collection of 1033 medical abstracts. For example we performed factorization
    on term-by-document matrix by constructing a matrix of shape 4765 (terms) x 1033 (documents). Original number
    of terms is 16017, the reduced number is a result of text preprocessing, namely removing stop words, too short words, 
    words that appear 2 times or less in the corpus and words that appear 50 times or more.

    .. note:: Medlars data set of medical abstracts used in this example is not included in the `datasets` and need to be
      downloaded. Download links are listed in the ``datasets``. Download compressed version of document text. To run the example, 
      the extracted Medlars data set must exist in the ``Medlars`` directory under ``datasets``. 
      
    Example of medical abstract::
        
        autolysis of bacillus subtilis by glucose depletion .                   
        in cultures in minimal medium, rapid lysis of cells of bacillus       
        subtilis was observed as soon as the carbon source, e.g. glucose, had   
        been completely consumed . the cells died and ultraviolet-absorbing     
        material was excreted in the medium . the results suggest that the cells
        lyse because of the presence of autolytic enzymes . in the presence of  
        glucose the damage to the cell wall caused by these enzymes is repaired 
        immediately . 
    
    Because of the nature of analysis, the resulting data matrix is very sparse. Therefore we use ``scipy.sparse`` matrix
    formats in factorization. This results in lower space consumption. Using, Standard NMF - Divergence, fitted
    factorization model is sparse as well, according to [Hoyer2004]_ measure of sparseness, the basis matrix has
    sparseness of 0.641 and the mixture matrix 0.863.
    
    .. note:: This sparseness 
              measure quantifies how much energy of a vector is packed into only few components. The sparseness of a vector
              is a real number in [0, 1]. Sparser vector has value closer to 1. The measure is 1 iff vector contains single
              nonzero component and the measure is equal to 0 iff all components are equal. Sparseness of a matrix is 
              the mean sparseness of its column vectors.
    
    The configuration of this example is sparse data matrix with Standard NMF - Divergence factorization method using 
    Random Vcol algorithm for initialization and rank 15 (the number of hidden topics). 
    
    Because of nonnegativity constraints, NMF has impressive benefits in terms of interpretation of its factors. In text
    processing applications, factorization rank can be considered the number of hidden topics present in the document
    collection. The basis matrix becomes a term-by-topic matrix whose columns are the basis vectors. Similar interpretation
    holds for the other factor, mixture matrix. Mixture matrix is a topic-by-document matrix with sparse nonnegative 
    columns. Element j of column 1 of mixture matrix measures the strength to which topic j appears in document 1. 
    
    .. figure:: /images/documents_basisW1.png
       :scale: 60 %
       :alt: Highest weighted terms in basis vector W1. 
       :align: center

       Interpretation of NMF - Divergence basis vectors on Medlars data set. Highest weighted terms in basis vector W1. The nonzero elements of column 1
       of W (W1), which is sparse and nonnegative, correspond to particular terms. By considering the highest weighted terms in this vector, 
       we can assign a label or topic to basis vector W1. As the NMF allows user the ability to interpret the basis vectors, a user might
       attach the label ``liver`` to basis vector W1. As a note, the term in 10th place, `viii`, is not a Roman numeral but
       instead `Factor viii`, an essential blood clotting factor also known as anti-hemophilic factor. It has been found
       to be synthesized and released into the bloodstream by the vascular, glomerular and tubular endothelium and 
       the sinusoidal cells of the ``liver``.
       
       
    .. figure:: /images/documents_basisW4.png
       :scale: 60 %
       :alt: Highest weighted terms in basis vector W4. 
       :align: center

       Interpretation of NMF basis vectors on Medlars data set. Highest weighted terms in basis vector W4. 
       
       
    .. figure:: /images/documents_basisW13.png
       :scale: 60 %
       :alt: Highest weighted terms in basis vector W13. 
       :align: center

       Interpretation of NMF basis vectors on Medlars data set. Highest weighted terms in basis vector W13. 
       
       
    .. figure:: /images/documents_basisW15.png
       :scale: 60 %
       :alt: Highest weighted terms in basis vector W15. 
       :align: center

       Interpretation of NMF basis vectors on Medlars data set. Highest weighted terms in basis vector W15. 
    
    To run the example simply type::
        
        python documents.py
        
    or call the module's function::
    
        import nimfa.examples
        nimfa.examples.documents.run()
        
    .. note:: This example uses ``matplotlib`` library for producing visual interpretation of NMF basis vectors on Medlars
              data set.
"""

from os.path import dirname, abspath
from os.path import join
from warnings import warn

import scipy.sparse as sp
import numpy as np

import nimfa

try:
    import matplotlib.pylab as plb
except ImportError as exc:
    warn("Matplotlib must be installed to run Documents example.")


def run():
    """Run NMF - Divergence on the Medlars data set."""
    V, term2idx, idx2term = read()
    V, term2idx, idx2term = preprocess(V, term2idx, idx2term)
    W, _ = factorize(V)
    plot(W, idx2term)


def factorize(V):
    """
    Perform NMF - Divergence factorization on the sparse Medlars data matrix. 
    
    Return basis and mixture matrices of the fitted factorization model. 
    
    :param V: The Medlars data matrix. 
    :type V: `scipy.sparse.csr_matrix`
    """
    nmf = nimfa.Nmf(V, seed="random_vcol", rank=12, max_iter=15, update="divergence",
                    objective="div")
    print("Algorithm: %s\nInitialization: %s\nRank: %d" % (nmf, nmf.seed, nmf.rank))
    fit = nmf()
    sparse_w, sparse_h = fit.fit.sparseness()
    print("""Stats:
            - iterations: %d
            - KL Divergence: %5.3f
            - Euclidean distance: %5.3f
            - Sparseness basis: %5.3f, mixture: %5.3f""" % (fit.fit.n_iter, fit.distance(), fit.distance(metric='euclidean'), sparse_w, sparse_h))
    return fit.basis(), fit.coef()


def read():
    """
    Read medical abstracts data from Medlars data set. 
    
    Construct a term-by-document matrix. This matrix is sparse, therefore ``scipy.sparse`` format is used. For construction
    LIL sparse format is used, which is an efficient structure for constructing sparse matrices incrementally. 
    
    Return the Medlars sparse data matrix in LIL format, term-to-index `dict` translator and index-to-term 
    `dict` translator. 
    """
    print("Read Medlars medical abstracts data set")
    dir = join(dirname(dirname(abspath(__file__))), "datasets", "Medlars", "med.all")
    doc = open(dir)
    V = sp.lil_matrix((16017, 1033))
    term2idx = {}
    idx2term = {}
    n_free = 0
    line = doc.readline()
    for abstract in range(1033):
        ii = int(line.split()[1])
        # omit .W char
        doc.readline()
        line = doc.readline()
        while line != ".I " + str(ii + 1) and line != "":
            for term in line.split():
                term = term.strip().replace(',', '').replace('.', '')
                if term not in term2idx:
                    term2idx[term] = n_free
                    idx2term[n_free] = term
                    n_free += 1
                V[term2idx[term], ii - 1] += 1
            line = doc.readline().strip()
    return V, term2idx, idx2term


def preprocess(V, term2idx, idx2term):
    """
    Preprocess Medlars data matrix. Remove stop words, digits, too short words, words that appear 2 times or less 
    in the corpus and words that appear 50 times or more.
    
    Return preprocessed term-by-document sparse matrix in CSR format. Returned matrix's shape is 4765 (terms) x 1033 (documents). 
    The sparse data matrix is converted to CSR format for fast arithmetic and matrix vector operations. Return
    updated index-to-term and term-to-index translators.
    
    :param V: The Medlars data matrix. 
    :type V: `scipy.sparse.lil_matrix`
    :param term2idx: Term-to-index translator.
    :type term2idx: `dict`
    :param idx2term: Index-to-term translator.
    :type idx2term: `dict`
    """
    print("Data preprocessing")
    # remove stop words, digits, too short words
    rem = set()
    for term in term2idx:
        if term in stop_words or len(term) <= 2 or str.isdigit(term):
            rem.add(term2idx[term])
    # remove words that appear two times or less in corpus
    V = V.tocsr()
    for r in range(V.shape[0]):
        if V[r, :].sum() <= 2 or V[r,:].sum() >= 50:
            rem.add(r)
    retain = set(range(V.shape[0])).difference(rem)
    n_free = 0
    V1 = sp.lil_matrix((V.shape[0] - len(rem), 1033))
    for r in retain:
        term2idx[idx2term[r]] = n_free
        idx2term[n_free] = idx2term[r]
        V1[n_free, :] = V[r,:] 
        n_free += 1
    return V1.tocsr(), term2idx, idx2term


def plot(W, idx2term):
    """
    Plot the interpretation of NMF basis vectors on Medlars data set. 
    
    :param W: Basis matrix of the fitted factorization model.
    :type W: `scipy.sparse.csr_matrix`
    :param idx2term: Index-to-term translator.
    :type idx2term: `dict`
    """
    print("Plot highest weighted terms in basis vectors")
    for c in range(W.shape[1]):
        if sp.isspmatrix(W):
            top10 = np.argsort(np.asarray(W[:, c].todense()).flatten())[-10:]
            val = W[top10, c].todense()
        else:
            top10 = np.argsort(np.asarray(W[:, c]).flatten())[-10:]
            val = W[top10, c]
        plb.figure(c + 1)
        plb.barh(np.arange(10) + .5, val, color="yellow", align="center")
        plb.yticks(np.arange(10) + .5, [idx2term[idx] for idx in top10][::-1])
        plb.xlabel("Weight")
        plb.ylabel("Term")
        plb.title("Highest Weighted Terms in Basis Vector W%d" % (c + 1))
        plb.grid(True)
        plb.savefig("documents_basisW%d.png" % (c + 1), bbox_inches="tight")


stop_words = [
    "a", "able", "about", "across", "after", "all", "almost", "also", "am", "among", "an",
    "and", "any", "are", "as", "at", "be", "because", "been", "but", "by", "can", "cannot",
    "could", "dear", "did", "do", "does", "either", "else", "ever", "every", "for", "from",
    "get", "got", "had", "has", "have", "he", "her", "hers", "him", "his", "how", "however",
    "i", "if", "in", "into", "is", "it", "its", "just", "least", "let", "like", "likely",
    "may", "me", "might", "most", "must", "my", "neither", "no", "nor", "not", "of", "off",
    "often", "on", "only", "or", "other", "our", "own", "rather", "said", "say", "says", "she",
    "should", "since", "so", "some", "than", "that", "the", "their", "them", "then", "there",
    "these", "they", "this", "tis", "to", "too", "twas", "us", "wants", "was", "we", "were",
    "what", "when", "where", "which", "while", "who", "whom", "why", "will", "with", "would",
    "yet", "you", "your", ".", " ", "1", "2", "3", "4", "5", "6", "7", "8", "9", "0", "during",
    "changes", "(1)", "(2)", "(3)", "(4)", "(5)", "(6)", "(7)", "(8)", "(9)", "usually", "involved",
    "labeled"]


if __name__ == "__main__":
    """Run the Medlars example."""
    run()



"""Test examples provided by nimfa"""

import numpy as np

from nimfa.examples import synthetic
from nimfa.examples import orl_images
from nimfa.examples import all_aml
from nimfa.examples import medulloblastoma
from nimfa.examples import recommendations
from nimfa.examples import cbcl_images


import pytest

synthetic_runs = [getattr(synthetic, f) for f in dir(synthetic) if f.startswith('run_')]

@pytest.mark.medium
@pytest.mark.parametrize('synth_run', synthetic_runs)
def test_synthetic(synth_run, monkeypatch):
    ran_funcs = []
    def check_synthetic_result(fit, idx=None):
        """Basic checks that synthetic results are 'Ok'"""
        ran_funcs.append(1)
        # typically tests pass with > .75 but because of still present inherent
        # randomizations results vary. So to be on a safe side for now set a very
        # liberal threshold
        # TODO:  make runs deterministic
        assert fit.summary(idx)['evar'] > 0.62

    # Setup data exactly the same way as in synthetic example
    prng = np.random.RandomState(42)
    # construct target matrices
    V = prng.rand(20, 30)
    V1 = prng.rand(20, 25)

    monkeypatch.setattr(synthetic, 'print_info', check_synthetic_result)
    if synth_run is getattr(synthetic, 'run_snmnmf'):
        synth_run(V, V1)
    else:
        synth_run(V)
    # for paranoid monkeypatching people, to make sure that the patch worked
    assert ran_funcs != [], "we must have ran our check"

#Check if the example datasets run
@pytest.mark.slow
def test_orl():
    orl_images.run()

@pytest.mark.slow
def test_aml():
    all_aml.run()

@pytest.mark.slow
def test_medulloblastoma():
    medulloblastoma.run()    

#@pytest.mark.slow
#def test_recommendations():
#    recommendations.run()    

@pytest.mark.slow
def test_cbcl():
    cbcl_images.run()    



import numpy as np

import nimfa

V = np.random.rand(40, 100)
snmf = nimfa.Snmf(V, seed="random_c", rank=10, max_iter=12, version='r', eta=1.,
                  beta=1e-4, i_conv=10, w_min_change=0)
snmf_fit = snmf()




"""
    ##############################################
    Recommendations (``examples.recommendations``)
    ##############################################
    
    In this examples of collaborative filtering we consider movie recommendation using common MovieLens data set. It 
    represents typical cold start problem. A recommender system compares the user's profile to reference
    characteristics from the user's social environment. In the collaborative filtering approach, the recommender
    system identify users who share the same preference with the active user and propose items which the like-minded
    users favoured (and the active user has not yet seen).     
    
    We used the MovieLens 100k data set in this example. This data set consists of 100 000 ratings (1-5) from 943
    users on 1682 movies. Each user has rated at least 20 movies. Simple demographic info for the users is included. 
    Factorization is performed on a split data set as provided by the collector of the data. The data is split into 
    two disjoint sets each consisting of training set and a test set with exactly 10 ratings per user. 
    
    It is common that matrices in the field of recommendation systems are very sparse (ordinary user rates only a small
    fraction of items from the large items' set), therefore ``scipy.sparse`` matrix formats are used in this example. 
    
    The configuration of this example is SNMF/R factorization method using Random Vcol algorithm for initialization. 
    
    .. note:: MovieLens movies' rating data set used in this example is not included in the `datasets` and need to be
      downloaded. Download links are listed in the ``datasets``. Download compressed version of the MovieLens 100k. 
      To run the example, the extracted data set must exist in the ``MovieLens`` directory under ``datasets``. 
      
    .. note:: No additional knowledge in terms of ratings' timestamps, information about items and their
       genres or demographic information about users is used in this example. 
      
    To run the example simply type::
        
        python recommendations.py
        
    or call the module's function::
    
        import nimfa.examples
        nimfa.examples.recommendations.run()
        
    .. note:: This example uses ``matplotlib`` library for producing visual interpretation of the RMSE error measure. 
    
"""

from os.path import dirname, abspath
from os.path import join
from warnings import warn

import numpy as np

import nimfa


try:
    import matplotlib.pylab as plb
except ImportError as exc:
    warn("Matplotlib must be installed to run Recommendations example.")


def run():
    """
    Run SNMF/R on the MovieLens data set.
    
    Factorization is run on `ua.base`, `ua.test` and `ub.base`, `ub.test` data set. This is MovieLens's data set split 
    of the data into training and test set. Both test data sets are disjoint and with exactly 10 ratings per user
    in the test set. 
    """
    for data_set in ['ua', 'ub']:
        V = read(data_set)
        W, H = factorize(V)
        rmse(W, H, data_set)


def factorize(V):
    """
    Perform SNMF/R factorization on the sparse MovieLens data matrix. 
    
    Return basis and mixture matrices of the fitted factorization model. 
    
    :param V: The MovieLens data matrix. 
    :type V: `numpy.matrix`
    """
    snmf = nimfa.Snmf(V, seed="random_vcol", rank=30, max_iter=30, version='r', eta=1.,
                      beta=1e-4, i_conv=10, w_min_change=0)
    print("Algorithm: %s\nInitialization: %s\nRank: %d" % (snmf, snmf.seed, snmf.rank))
    fit = snmf()
    sparse_w, sparse_h = fit.fit.sparseness()
    print("""Stats:
            - iterations: %d
            - Euclidean distance: %5.3f
            - Sparseness basis: %5.3f, mixture: %5.3f""" % (fit.fit.n_iter,
                                                            fit.distance(metric='euclidean'),
                                                            sparse_w, sparse_h))
    return fit.basis(), fit.coef()


def read(data_set):
    """
    Read movies' ratings data from MovieLens data set. 
    
    :param data_set: Name of the split data set to be read.
    :type data_set: `str`
    """
    print("Read MovieLens data set")
    fname = join(dirname(dirname(abspath(__file__))), "datasets", "MovieLens", "%s.base" % data_set)
    V = np.ones((943, 1682)) * 2.5
    for line in open(fname):
        u, i, r, _ = list(map(int, line.split()))
        V[u - 1, i - 1] = r
    return V


def rmse(W, H, data_set):
    """
    Compute the RMSE error rate on MovieLens data set.
    
    :param W: Basis matrix of the fitted factorization model.
    :type W: `numpy.matrix`
    :param H: Mixture matrix of the fitted factorization model.
    :type H: `numpy.matrix`
    :param data_set: Name of the split data set to be read. 
    :type data_set: `str`
    """
    fname = join(dirname(dirname(abspath(__file__))), "datasets", "MovieLens", "%s.test" % data_set)
    rmse = []
    for line in open(fname):
        u, i, r, _ = list(map(int, line.split()))
        sc = max(min((W[u - 1, :] * H[:, i - 1])[0, 0], 5), 1)
        rmse.append((sc - r) ** 2)
    print("RMSE: %5.3f" % np.sqrt(np.mean(rmse)))


if __name__ == "__main__":
    """Run the Recommendations example."""
    run()




"""
    This package contains implementations of the MF algorithms and seeding methods.
"""

from . import factorization
from . import seeding


def list_mf_methods():
    """Return list of implemented MF methods."""
    return [name for name in factorization.methods]


def list_seeding_methods():
    """Return list of implemented seeding methods."""
    return [name for name in seeding.methods]




"""
    ##########################
    Nmf_mm (``models.nmf_mm``)
    ##########################
"""

from .nmf import *


class Nmf_mm(Nmf):
    """
    Implementation of the alternative model to manage factorizations that follow
    NMF nonstandard model. This modification is required by the Multiple NMF
    algorithms (e. g. SNMNMF [Zhang2011]_). The Multiple NMF algorithms modify the
    standard divergence or Euclidean based NMF methods by introducing multiple
    mixture (coefficients) matrices and target matrices.
     
    It is the underlying model of matrix factorization and provides structure of
    modified standard NMF model.
    
    .. attribute:: W
        
        Basis matrix -- the first matrix factor in the multiple NMF model
        
    .. attribute:: H
    
        Mixture matrix -- the second matrix factor in the multiple NMF model (coef0)
    
    .. attribute:: H1
    
        Mixture matrix -- the second matrix factor in the multiple NMF model (coef1)
        
    .. attribute:: V1
    
        Target matrix, the matrix for the MF method to estimate.
        
    The interpretation of the basis and mixture matrix is such as in the standard NMF model. 
    
    Multiple NMF specify more than one target matrix. In that case target matrices are
    passed as tuples. Internally, additional attributes with names following Vn pattern
    are created, where n is the consecutive index of target matrix. Zero index is omitted
    (there are V, V1, V2, V3, etc. matrices and then H, H1, H2, etc. and W, W1, W2, etc.
    respectively).
    
    Currently, in implemented multiple NMF method V, V1 and H, H1 are needed. There is only
    one basis matrix (W).
    """
    def __init__(self, params):
        """
        Construct factorization model that manages multiple NMF models.
        
        :param params: MF runtime and algorithm parameters and options. For detailed
           explanation of the general model parameters see :mod:`mf_run`. For algorithm
           specific model options see documentation of chosen factorization method.
        :type params: `dict`
        """
        self.model_name = "mm"
        Nmf.__init__(self, params)
        if sp.isspmatrix(self.V) and (self.V.data < 0).any() or not sp.isspmatrix(self.V) and (self.V < 0).any():
            raise utils.MFError("The input matrix contains negative elements.")
        if sp.isspmatrix(self.V1) and (self.V1.data < 0).any() or not sp.isspmatrix(self.V1) and (self.V1 < 0).any():
            raise utils.MFError("The input matrix contains negative elements.")

    def basis(self):
        """Return the matrix of basis vectors."""
        return self.W

    def target(self, idx):
        """
        Return the target matrix to estimate.
        
        :param idx: Name of the matrix (coefficient) matrix.
        :type idx: `str` with values 'coef' or 'coef1' (`int` value of 0 or 1 respectively) 
        """
        if idx == 'coef' or idx == 0:
            return self.V
        elif idx == 'coef1' or idx == 1:
            return self.V1
        raise utils.MFError("Unknown specifier for the target matrix.")

    def coef(self, idx):
        """
        Return the matrix of mixture coefficients.
        
        :param idx: Name of the matrix (coefficient) matrix.
        :type idx: `str` with values 'coef' or 'coef1' (`int` value of 0 or 1 respectively) 
        """
        if idx == 'coef' or idx == 0:
            return self.H
        elif idx == 'coef1' or idx == 1:
            return self.H1
        raise utils.MFError("Unknown specifier for the mixture matrix.")

    def fitted(self, idx):
        """
        Compute the estimated target matrix according to the nonsmooth NMF algorithm model.
        
        :param idx: Name of the matrix (coefficient) matrix.
        :type idx: `str` with values 'coef' or 'coef1' (`int` value of 0 or 1 respectively) 
        """
        if idx == 'coef' or idx == 0:
            return dot(self.W, self.H)
        elif idx == 'coef1' or idx == 1:
            return dot(self.W, self.H1)
        raise utils.MFError("Unknown specifier for the mixture matrix.")

    def distance(self, metric='euclidean', idx=None):
        """
        Return the loss function value.

        :param distance: Specify distance metric to be used. Possible are Euclidean
           and Kullback-Leibler (KL) divergence. Strictly, KL is not a metric.
        :type distance: `str` with values 'euclidean' or 'kl'

        :param idx: Name of the matrix (coefficient) matrix.
        :type idx: `str` with values 'coef' or 'coef1' (`int` value of 0 or 1 respectively) 
        """
        if idx == 'coef' or idx == 0:
            H = self.H
            V = self.V
        elif idx == 'coef1' or idx == 1:
            H = self.H1
            V = self.V1
        else:
            raise utils.MFError("Unknown specifier for the mixture matrix.")
        if metric.lower() == 'euclidean':
            return power(V - dot(self.W, H), 2).sum()
        elif metric.lower() == 'kl':
            Va = dot(self.W, H)
            return (multiply(V, sop(elop(V, Va, div), op=np.log)) - V + Va).sum()
        else:
            raise utils.MFError("Unknown distance metric.")

    def residuals(self, idx):
        """
        Return residuals matrix between the target matrix and its multiple NMF estimate.
        
        :param idx: Name of the matrix (coefficient) matrix.
        :type idx: `str` with values 'coef' or 'coef1' (`int` value of 0 or 1 respectively) 
        """
        if idx == 'coef' or idx == 0:
            H = self.H
            V = self.V
        elif idx == 'coef1' or idx == 1:
            H = self.H1
            V = self.V1
        else:
            raise utils.MFError("Unknown specifier for the mixture matrix.")
        return V - dot(self.W, H)




"""    
    This package contains implementations of linear algebra operations for sparse and dense matrices and
    utils for convenient operation of nimfa library.
"""

from . import linalg
from . import utils



import numpy as np

import nimfa

V = np.random.rand(40, 100)
bd = nimfa.Bd(V, seed="random_c", rank=10, max_iter=12, alpha=np.zeros((V.shape[0], 10)),
              beta=np.zeros((10, V.shape[1])), theta=.0, k=.0, sigma=1., skip=100, stride=1,
              n_w=np.zeros((10, 1)), n_h=np.zeros((10, 1)), n_sigma=False)
bd_fit = bd()




"""    
    This module lists data sets needed to run Nimfa's use cases.

    .. note:: Relevant code for data processing is provided within each example. To run an example download the relevant data and put it in ``nimfa/datasets`` directory.
    
    Data sets:
    
        #. Leukemia data set; `leukemia gene expressions`_, `leukemia samples`_, `leukemia genes`_.
        #. Medulloblastoma data set; `medulloblastoma gene expressions`_, `medulloblastoma samples`_, `medulloblastoma genes`_.
        #. `CBCL face images`_ database.
        #. `ORL face images`_ database.
        #. `MovieLens 100k`_ data set.
        #. `Medlars`_ data set.
        #. `S. cerevisiae`_ MIPS Functional Catalogue data set.
        
    .. _leukemia gene expressions: http://portals.broadinstitute.org/cgi-bin/cancer/publications/pub_paper.cgi?mode=view&paper_id=89
    .. _leukemia samples: http://portals.broadinstitute.org/cgi-bin/cancer/publications/pub_paper.cgi?mode=view&paper_id=89
    .. _leukemia genes: http://portals.broadinstitute.org/cgi-bin/cancer/publications/pub_paper.cgi?mode=view&paper_id=89
    .. _medulloblastoma gene expressions: http://portals.broadinstitute.org/cgi-bin/cancer/publications/pub_paper.cgi?mode=view&paper_id=89
    .. _medulloblastoma samples: http://portals.broadinstitute.org/cgi-bin/cancer/publications/pub_paper.cgi?mode=view&paper_id=89
    .. _medulloblastoma genes: http://portals.broadinstitute.org/cgi-bin/cancer/publications/pub_paper.cgi?mode=view&paper_id=89
    .. _CBCL face images: http://cbcl.mit.edu/cbcl/software-datasets/FaceData2.html
    .. _ORL face images: http://www.cl.cam.ac.uk/research/dtg/attarchive/facedatabase.html
    .. _MovieLens 100k: http://www.grouplens.org/node/12
    .. _Medlars: http://web.eecs.utk.edu/research/lsi/
    .. _S. cerevisiae: http://dtai.cs.kuleuven.be/clus/hmc-ens/
        
"""




"""
#####################################
Psmf (``methods.factorization.psmf``)
#####################################

**Probabilistic Sparse Matrix Factorization (PSMF)** [Dueck2005]_, [Dueck2004]_. 

PSMF allows for varying levels of sensor noise in the
data, uncertainty in the hidden prototypes used to explain the data and
uncertainty as to the prototypes selected to explain each data vector stacked in
target matrix (V).

This technique explicitly maximizes a lower bound on the log-likelihood of the
data under a probability model. Found sparse encoding can be used for a variety
of tasks, such as functional prediction, capturing functionally relevant
hidden factors that explain gene expression data and visualization. As this
algorithm computes probabilities rather than making hard decisions, it can be
shown that a higher data log-likelihood is obtained than from the
versions (iterated conditional modes) that make hard decisions [Srebro2001]_.

Given a target matrix (V [n, m]), containing n m-dimensional data points, basis
matrix (factor loading matrix) (W) and mixture matrix (matrix of hidden factors)
(H) are found under a structural sparseness constraint that each row of W
contains at most N (of possible factorization rank number) non-zero entries.
Intuitively, this corresponds to explaining each row vector of V as a linear
combination (weighted by the corresponding row in W) of a small subset
of factors given by rows of H. This framework includes simple clustering by
setting N = 1 and ordinary low-rank approximation N = factorization rank as
special cases.

A probability model presuming Gaussian sensor noise in V (V = WH + noise) and
uniformly distributed factor assignments is constructed. Factorized variational
inference method is used to perform tractable inference on the latent variables
and account for noise and uncertainty. The number of factors, r_g, contributing
to each data point is multinomially distributed such that P(r_g = n) = v_n,
where v is a user specified N-vector. PSMF model estimation using factorized
variational inference has greater computational complexity than basic NMF
methods [Dueck2004]_.

Example of usage of PSMF for identifying gene transcriptional modules from gene
expression data is described in [Li2007]_.

.. literalinclude:: /code/snippet_psmf.py

"""

import numbers

from nimfa.models import *
from nimfa.utils import *
from nimfa.utils.linalg import *

__all__ = ['Psmf']


class Psmf(nmf_std.Nmf_std):
    """
    :param V: The target matrix to estimate.
    :type V: Instance of the :class:`scipy.sparse` sparse matrices types,
       :class:`numpy.ndarray`, :class:`numpy.matrix` or tuple of instances of
       the latter classes.

    :param seed: Specify method to seed the computation of a factorization. If
       specified :param:`W` and :param:`H` seeding must be None. If neither seeding
       method or initial fixed factorization is specified, random initialization is
       used.
    :type seed: `str` naming the method or :class:`methods.seeding.nndsvd.Nndsvd`
       or None

    :param W: Specify initial factorization of basis matrix W. Default is None.
       When specified, :param:`seed` must be None.
    :type W: :class:`scipy.sparse` or :class:`numpy.ndarray` or
       :class:`numpy.matrix` or None

    :param H: Specify initial factorization of mixture matrix H. Default is None.
       When specified, :param:`seed` must be None.
    :type H: Instance of the :class:`scipy.sparse` sparse matrices types,
       :class:`numpy.ndarray`, :class:`numpy.matrix`, tuple of instances of the
       latter classes or None

    :param rank: The factorization rank to achieve. Default is 30.
    :type rank: `int`

    :param n_run: It specifies the number of runs of the algorithm. Default is
       1. If multiple runs are performed, fitted factorization model with the
       lowest objective function value is retained.
    :type n_run: `int`

    :param callback: Pass a callback function that is called after each run when
       performing multiple runs. This is useful if one wants to save summary
       measures or process the result before it gets discarded. The callback
       function is called with only one argument :class:`models.mf_fit.Mf_fit` that
       contains the fitted model. Default is None.
    :type callback: `function`

    :param callback_init: Pass a callback function that is called after each
       initialization of the matrix factors. In case of multiple runs the function
       is called before each run (more precisely after initialization and before
       the factorization of each run). In case of single run, the passed callback
       function is called after the only initialization of the matrix factors.
       This is useful if one wants to obtain the initialized matrix factors for
       further analysis or additional info about initialized factorization model.
       The callback function is called with only one argument
       :class:`models.mf_fit.Mf_fit` that (among others) contains also initialized
       matrix factors. Default is None.
    :type callback_init: `function`

    :param track_factor: When :param:`track_factor` is specified, the fitted
        factorization model is tracked during multiple runs of the algorithm. This
        option is taken into account only when multiple runs are executed
        (:param:`n_run` > 1). From each run of the factorization all matrix factors
        are retained, which can be very space consuming. If space is the problem
        setting the callback function with :param:`callback` is advised which is
        executed after each run. Tracking is useful for performing some quality or
        performance measures (e.g. cophenetic correlation, consensus matrix,
        dispersion). By default fitted model is not tracked.
    :type track_factor: `bool`

    :param track_error: Tracking the residuals error. Only the residuals from
        each iteration of the factorization are retained. Error tracking is not
        space consuming. By default residuals are not tracked and only the final
        residuals are saved. It can be used for plotting the trajectory of the
        residuals.
    :type track_error: `bool`

    :param prior: The prior on the number of factors explaining each vector. It
       should be a positive row vector. The ``prior`` can be passed as a
       list, formatted as prior = [P(r_g = 1), P(r_g = 2), ... P(r_q = N)] or as a
       scalar N, in which case uniform prior is taken, prior = 1. / (1:N),
       reflecting no knowledge about the distribution and giving equal preference to
       all values of a particular r_g. Default value for :param:`prior` is
       factorization rank.
    :type prior: `list` or `float`

    **Stopping criterion**

    Factorization terminates if any of specified criteria is satisfied.

    :param max_iter: Maximum number of factorization iterations. Note that the
       number of iterations depends on the speed of method convergence. Default
       is 30.
    :type max_iter: `int`

    :param min_residuals: Minimal required improvement of the residuals from the
       previous iteration. They are computed between the target matrix and its MF
       estimate using the objective function associated to the MF algorithm.
       Default is None.
    :type min_residuals: `float`

    :param test_conv: It indicates how often convergence test is done. By
       default convergence is tested each iteration.
    :type test_conv: `int`
    """
    def __init__(self, V, seed=None, W=None, H=None, H1=None,
                 rank=30, max_iter=30, min_residuals=1e-5, test_conv=None,
                 n_run=1, callback=None, callback_init=None, track_factor=False,
                 track_error=False, prior=20, **options):
        self.name = "psmf"
        self.aseeds = ["none"]
        nmf_std.Nmf_std.__init__(self, vars())
        self.test_conv = 5 if not self.test_conv else self.test_conv
        if isinstance(self.prior, numbers.Real):
            self.prior = np.ones(int(self.prior)) / self.prior
        self.tracker = mf_track.Mf_track() if self.track_factor and self.n_run > 1 \
                                              or self.track_error else None

    def factorize(self):
        """
        Compute matrix factorization.
         
        Return fitted factorization model.
        """
        self.N = len(self.prior)
        sm = sum(self.prior)
        self.prior = np.array([p / sm for p in self.prior])
        self.eps = 1e-5
        if sp.isspmatrix(self.V):
            self.V = self.V.todense()

        for run in range(self.n_run):
            # initialize P and Q distributions
            # internal computation is done with numpy arrays as n-(n >
            # 2)dimensionality is needed
            if sp.isspmatrix(self.V):
                self.W = self.V.__class__(
                    (self.V.shape[0], self.rank), dtype='d')
                self.H = self.V.__class__(
                    (self.rank, self.V.shape[1]), dtype='d')
            else:
                self.W = np.mat(np.zeros((self.V.shape[0], self.rank)))
                self.H = np.mat(np.zeros((self.rank, self.V.shape[1])))
            self.s = np.zeros((self.V.shape[0], self.N), int)
            self.r = np.zeros((self.V.shape[0], 1), int)
            self.psi = np.array(std(self.V, axis=1, ddof=0))
            self.lamb = abs(np.tile(np.sqrt(self.psi), (1, self.rank))
                            * np.random.randn(self.V.shape[0], self.rank))
            self.zeta = np.random.rand(self.rank, self.V.shape[1])
            self.phi = np.random.rand(self.rank, 1)
            self.sigma = np.random.rand(self.V.shape[0], self.rank, self.N)
            self.sigma = self.sigma / np.tile(self.sigma.sum(axis=1).reshape(
                (self.sigma.shape[0], 1, self.sigma.shape[2])), (1, self.rank, 1))
            self.rho = np.tile(self.prior, (self.V.shape[0], 1))
            self._cross_terms()
            p_obj = c_obj = sys.float_info.max
            best_obj = c_obj if run == 0 else best_obj
            iter = 0
            if self.callback_init:
                self.final_obj = c_obj
                self.n_iter = iter
                mffit = mf_fit.Mf_fit(self)
                self.callback_init(mffit)
            while self.is_satisfied(p_obj, c_obj, iter):
                p_obj = c_obj if not self.test_conv or iter % self.test_conv == 0 else p_obj
                self.update()
                iter += 1
                c_obj = self.objective(
                ) if not self.test_conv or iter % self.test_conv == 0 else c_obj
                if self.track_error:
                    self.tracker.track_error(run, c_obj)
            if self.callback:
                self.final_obj = c_obj
                self.n_iter = iter
                mffit = mf_fit.Mf_fit(self)
                self.callback(mffit)
            if self.track_factor:
                self.tracker.track_factor(
                    run, W=self.W, H=self.H, final_obj=c_obj, n_iter=iter)
            # if multiple runs are performed, fitted factorization model with
            # the lowest objective function value is retained
            if c_obj <= best_obj or run == 0:
                best_obj = c_obj
                self.n_iter = iter
                self.final_obj = c_obj
                mffit = mf_fit.Mf_fit(copy.deepcopy(self))

        mffit.fit.tracker = self.tracker
        return mffit

    def _cross_terms(self):
        """Initialize the major cached parameter."""
        outer_zeta = np.dot(self.zeta, self.zeta.T)
        self.cross_terms = {}
        for n1 in range(self.N):
            for n2 in range(n1 + 1, self.N):
                self.cross_terms[n1, n2] = np.zeros((self.V.shape[0], 1))
                for c in range(self.rank):
                    sigmat = np.tile((self.sigma[:, c, n2] * self.lamb[:, c]).reshape((self.lamb.shape[0], 1)), (1, self.zeta.shape[0]))
                    outer_zetat = np.tile(outer_zeta[c, :], (self.rho.shape[0], 1))
                    self.cross_terms[n1, n2] += (self.sigma[:, :, n1] * self.lamb * 
                                                 sigmat * outer_zetat).sum(axis = 1).reshape(self.rho.shape[0], 1)

    def is_satisfied(self, p_obj, c_obj, iter):
        """
        Compute the satisfiability of the stopping criteria based on stopping
        parameters and objective function value.
        
        Return logical value denoting factorization continuation. 
        
        :param p_obj: Objective function value from previous iteration. 
        :type p_obj: `float`
        :param c_obj: Current objective function value.
        :type c_obj: `float`
        :param iter: Current iteration number. 
        :type iter: `int`
        """
        if self.max_iter and self.max_iter <= iter:
            return False
        if self.test_conv and iter % self.test_conv != 0:
            return True
        if self.min_residuals and iter > 0 and p_obj - c_obj < self.min_residuals:
            return False
        if iter > 0 and c_obj > p_obj:
            return False
        return True

    def update(self):
        """Update basis and mixture matrix."""
        self._update_rho()
        self._update_phi()
        self._update_zeta()
        self._update_sigma()
        self._update_lamb()
        self._update_psi()

    def _update_psi(self):
        """Compute M-step and update psi."""
        t_p1 = np.array(multiply(self.V, self.V).sum(axis=1))
        self.psi = - \
            (np.tile(list(range(1, self.N)), (self.V.shape[0], 1)) * self.rho[:, 1:self.N]).sum(
                axis=1) * t_p1[:, 0]
        self.psi = self.psi.reshape((self.V.shape[0], 1))
        temp = np.zeros((self.V.shape[0], self.rank))
        for t in range(self.V.shape[1]):
            temp += (np.tile(self.__arr(self.V[:, t]), (1, self.rank)) - self.lamb * np.tile(
                self.zeta[:, t].T, (self.V.shape[0], 1))) ** 2 + self.lamb ** 2 * np.tile(self.phi.T, (self.V.shape[0], 1))
        for n in range(self.N):
            self.psi += (self.rho[:, n:self.N].sum(axis = 1) * (self.sigma[:, :, n] * temp).sum(axis = 1)).reshape(self.psi.shape)
        for n in range(self.N):
            for nn in range(n + 1, self.N):
                self.psi += (2 * self.rho[:, nn:self.N].sum(
                    axis=1) * self.cross_terms[n, nn].T[0]).reshape((self.V.shape[0], 1))
        self.psi /= self.V.shape[1]
        # heuristic: variances cannot go lower than epsilon
        self.psi = np.maximum(self.psi, self.eps)

    def _update_lamb(self):
        """Compute M-step and update lambda."""
        D = np.zeros((self.rank, 1))
        V = np.zeros((self.V.shape[0], self.rank))
        for t in range(self.V.shape[1]):
            D += self.zeta[:, t].reshape(
                (self.zeta.shape[0], 1)) ** 2 + self.phi
            V += dot(self.V[:, t], self.zeta[:, t].T)
        temp = np.zeros((self.V.shape[0], self.rank))
        for n in range(self.N):
            temp += np.tile((self.rho[:, n:self.N]).sum(axis = 1).reshape((self.rho.shape[0], 1)), (1, self.rank)) * self.sigma[:, :, n]
        V *= temp
        D = np.tile(D.T, (self.V.shape[0], 1)) * temp
        # heuristic: weak Gaussian prior on lambda for ill-conditioning
        # prevention
        D += self.eps
        for g in range(self.V.shape[0]):
            M = np.zeros((self.rank, self.rank))
            for n in range(self.N):
                for nn in range(n + 1, self.N):
                    M += np.dot(self.rho[g, nn:self.N].sum(axis = 0), np.dot(self.sigma[g, :, n].T, self.sigma[g,:, nn]))
            M = (M + M.T) * np.dot(self.zeta, self.zeta.T)
            self.lamb[g, :] = np.dot(V[g,:], np.linalg.inv(M + np.diag(D[g,:])))
        # heuristic:  negative mixing proportions not allowed
        self.lamb[self.lamb < 0] = 0
        self.W = sp.lil_matrix((self.V.shape[0], self.rank))
        for n in range(self.N):
            locs = (self.r >= n).ravel().nonzero()[0]
            if len(locs):
                locs = sub2ind(
                    (self.V.shape[0], self.rank), locs, self.s[locs, n])
                for l in locs:
                    self.W[l % self.V.shape[0], l // self.V.shape[0]] = self.lamb[
                        l % self.V.shape[0], l // self.V.shape[0]]
        self.W = self.W.tocsr()
        self._cross_terms()

    def _update_sigma(self):
        """Compute E-step and update sigma."""
        self.cross_terms = np.zeros((self.V.shape[0], self.rank, self.N))
        for cc in range(self.rank):
            t_c1 = np.tile(self.sigma[:, cc, :].reshape((self.sigma.shape[0], 1, self.sigma.shape[2])), (1, self.rank, 1))
            t_c2 = np.tile(np.dot(self.zeta[cc, :], self.zeta.T), (self.V.shape[0], 1))
            t_c3 = np.tile((self.lamb * np.tile(self.lamb[:, cc].reshape((self.lamb.shape[0], 1)), (1, self.rank)) * t_c2).reshape(
                t_c2.shape[0], t_c2.shape[1], 1), (1, 1, self.N))
            self.cross_terms += t_c1 * t_c3
        self.sigma = np.zeros(self.sigma.shape)
        for t in range(self.V.shape[1]):
            t_s1 = np.tile(self.__arr(self.V[:, t]), (1, self.rank)) - self.lamb * np.tile(
                self.zeta[:, t].T, (self.V.shape[0], 1))
            t_s2 = t_s1 ** 2 + self.lamb ** 2 * \
                np.tile(self.phi.T, (self.V.shape[0], 1))
            self.sigma -= 0.5 * \
                np.tile((t_s2 / np.tile(self.psi, (1, self.rank))).reshape(
                    t_s2.shape[0], t_s2.shape[1], 1), (1, 1, self.N))
        for n in range(self.N):
            for nn in range(self.N):
                if nn != n:
                    t_s1 = (1e-50 + self.rho[:, max(n, nn):self.N]).sum(
                        axis=1) / (1e-50 + self.rho[:, n:self.N]).sum(axis=1)
                    self.sigma[:, :, n] -= np.tile(t_s1.reshape(self.psi.shape) / self.psi, (1, self.rank)) * self.cross_terms[:,:, nn]        
        self.sigma = np.exp(self.sigma - np.tile(np.amax(self.sigma, 1).reshape(
            (self.sigma.shape[0], 1, self.sigma.shape[2])), (1, self.rank, 1)))
        self.sigma /= np.tile(self.sigma.sum(axis=1).reshape(
            (self.sigma.shape[0], 1, self.sigma.shape[2])), (1, self.rank, 1))
        self.cross_terms = self._cross_terms()
        self.s = np.argmax(self.sigma, axis=1)
        self.s = self.s.transpose([0, 1])

    def _update_zeta(self):
        """Compute E-step and update zeta."""
        M = np.zeros((self.rank, self.rank))
        V = np.zeros((self.rank, self.V.shape[1]))
        for cc in range(self.rank):
            for n in range(self.N):
                for nn in range(n + 1, self.N):
                    t_m1 = np.tile(self.rho[:, nn:self.N].sum(axis=1).reshape(
                        (self.psi.shape[0], 1)) / self.psi, (1, self.rank))
                    t_m2 = np.tile((self.lamb[:, cc] * self.sigma[:, cc, nn]).reshape(
                        (self.lamb.shape[0], 1)), (1, self.rank))
                    t_m =  t_m1 * self.lamb * self.sigma[:, :, n] * t_m2
                    M[cc, :] += t_m.sum(axis = 0)
        M += M.T
        temp = np.zeros((self.V.shape[0], self.rank))
        for n in range(self.N):
            temp += np.tile(self.rho[:, n:self.N].sum(axis = 1).reshape((self.rho.shape[0], 1)), (1, self.rank)) * self.sigma[:, :, n]
        M += np.diag(
            (self.lamb ** 2 / np.tile(self.psi, (1, self.rank)) * temp).sum(axis=0))
        for t in range(self.V.shape[1]):
            t_v = np.tile(
                self.__arr(self.V[:, t]) / self.psi, (1, self.rank)) * self.lamb * temp
            V[:, t] = t_v.sum(axis=0)
        self.zeta = np.linalg.solve(M + np.eye(self.rank), V)
        # heuristic: negative expression levels not allowed
        self.zeta[self.zeta < 0] = 0.
        self.H = sp.csr_matrix(self.zeta)

    def _update_phi(self):
        """Compute E-step and update phi."""
        self.phi = np.ones(self.phi.shape)
        for n in range(self.N):
            rho_tmp = np.tile(self.rho[:, n:self.N].sum(axis = 1).reshape(self.rho.shape[0], 1), (1, self.rank))
            t_phi = np.tile(self.psi, (1, self.rank)) * self.sigma[:, :, n] * rho_tmp
            self.phi += (self.lamb ** 2 / (t_phi + np.finfo(t_phi.dtype).eps)).sum(
                axis=0).reshape((self.phi.shape[0], 1))
        self.phi = 1. / self.phi
        # heuristic: variances cannot go lower than epsilon
        self.phi = np.maximum(self.phi, self.eps)

    def _update_rho(self):
        """Compute E-step and update rho."""
        self.rho = - (self.sigma * np.log(1e-50 + self.sigma)).sum(axis=1).reshape(
            self.sigma.shape[0], 1, self.sigma.shape[2]).cumsum(axis=2).transpose([0, 2, 1])
        self.rho = self.rho.reshape((self.rho.shape[0], self.rho.shape[1]))
        temp = np.zeros((self.V.shape[0], self.rank))
        for t in range(self.V.shape[1]):
            t_dot = np.array(
                np.dot(self.__arr(self.V[:, t]).reshape((self.V.shape[0], 1)), self.zeta[:, t].T.reshape((1, self.zeta.shape[0]))))
            temp -= 2 * self.lamb * t_dot + self.lamb ** 2 * \
                np.tile(
                    self.zeta[:, t].T ** 2 + self.phi.T, (self.V.shape[0], 1))
        for n in range(1, self.N):
            self.rho[:, n] -= 0.5 / self.psi[:, 0] * (self.sigma[:, :, 1:n].sum(axis = 2) * temp).sum(axis = 1)
            for n1 in range(n + 1):
                for n2 in range(n1 + 1, n + 1):
                    self.rho[:, n] -= (
                        1. / self.psi * self.cross_terms[n1, n2])[:, 0]
        t_rho = np.exp(
            self.rho - np.tile(np.amax(self.rho, 1).reshape((self.rho.shape[0], 1)), (1, self.N)))
        self.rho = np.tile((self.prior / self.rank) ** list(
            range(1, self.N + 1)), (self.V.shape[0], 1)) * t_rho
        self.rho = self.rho / \
            np.tile(self.rho.sum(axis=1).reshape(
                (self.rho.shape[0], 1)), (1, self.N))
        self.r = np.argmax(self.rho, axis=1)

    def objective(self):
        """Compute squared Frobenius norm of a target matrix and its NMF estimate."""
        R = self.V - dot(self.W, self.H)
        return power(R, 2).sum()

    def __arr(self, X):
        """Return dense vector X."""
        if sp.isspmatrix(X):
            return X.toarray()
        else:
            return np.array(X)

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name




"""
#################################
Fixed (``methods.seeding.fixed``)
#################################

Fixed factorization. This is the option to completely specify the initial
factorization by passing values for matrix factors.
"""

from nimfa.utils.linalg import *

__all__ = ['Fixed']


class Fixed(object):

    def __init__(self):
        self.name = "fixed"

    def _set_fixed(self, **factors):
        """Set initial factorization."""
        for k in list(factors.keys()):
            if factors[k] is not None:
                factors[k] = np.matrix(factors[k]) if not sp.isspmatrix(
                    factors[k]) else factors[k].copy()
            else:
                factors.pop(k)
        self.__dict__.update(factors)

    def initialize(self, V, rank, options):
        """
        Return fixed initialized matrix factors.
        
        :param V: Target matrix, the matrix for MF method to estimate. 
        :type V: One of the :class:`scipy.sparse` sparse matrices types or
                :class:`numpy.matrix`
        :param rank: Factorization rank. 
        :type rank: `int`
        :param options: Specify the algorithm and model specific options (e.g. initialization of
                extra matrix factor, seeding parameters).
                    
                Option ``idx`` represents the index of the coefficient matrix. The index is by default 0
                and corresponds to a factorization model with one mixture matrix (e.g. standard, nonsmooth
                model).
        :type options: `dict`
        """
        self.idx = options.get('idx', 0)
        if self.idx == 0:
            return self.W, self.H
        else:
            return self.W, getattr(self, 'H' + str(self.idx))

    def __repr__(self):
        return "fixed.Fixed()"

    def __str__(self):
        return self.name



import numpy as np
import scipy.sparse as sp

import nimfa

V = np.random.rand(40, 100)
V1 = np.random.rand(40, 200)
snmnmf = nimfa.Snmnmf(V=V, V1=V1, seed="random_c", rank=10, max_iter=12,
                      A=sp.csr_matrix((V1.shape[1], V1.shape[1])),
                      B=sp.csr_matrix((V.shape[1], V1.shape[1])), gamma=0.01,
                      gamma_1=0.01, lamb=0.01, lamb_1=0.01)
snmnmf_fit = snmnmf()




"""
    ####################################
    ORL Images (``examples.orl_images``)
    ####################################
    
    In this example of image processing we consider the image problem presented in [Hoyer2004]_. 
    
    We used the ORL face database composed of 400 images of size 112 x 92. There are 40 persons, 10 images per
    each person. The images were taken at different times, lighting and facial expressions. The faces are in 
    an upright position in frontal view, with a slight left-right rotation. In example we performed factorization
    on reduced face images by constructing a matrix of shape 2576 (pixels) x 400 (faces) and on original face
    images by constructing a matrix of shape 10304 (pixels) x 400 (faces). To avoid too large values, the data matrix is 
    divided by 100. Indeed, this division does not has any major impact on performance of the MF methods. 
    
    .. note:: The ORL face images database used in this example is included in the `datasets` and does not need to be
          downloaded. However, download links are listed in the ``datasets``. To run the example, the ORL face images
          must exist in the ``ORL_faces`` directory under ``datasets``. 
          
    We experimented with the Standard NMF - Euclidean, LSNMF and PSMF factorization methods to learn the basis images from the ORL database. The
    number of bases is 25. In [Lee1999]_ Lee and Seung showed that Standard NMF (Euclidean or divergence) found a parts-based
    representation when trained on face images from CBCL database. However, applying NMF to the ORL data set, in which images
    are not as well aligned, a global decomposition emerges. To compare, this example applies different MF methods to the face 
    images data set. Applying MF methods with sparseness constraint, namely PSMF, the resulting bases are not global, but instead
    give spatially localized representations, as can be seen from the figure. Similar conclusions are published in [Hoyer2004]_.
    Setting a high sparseness value for the basis images results in a local representation. 
    
    
    .. note:: It is worth noting that sparseness constraints do not always lead to local solutions. Global solutions can 
              be obtained by forcing low sparseness on basis matrix and high sparseness on coefficient matrix - forcing 
              each coefficient to represent as much of the image as possible. 
          
          
    .. figure:: /images/orl_faces_500_iters_large_LSNMF.png
       :scale: 70 %
       :alt: Basis images of LSNMF obtained after 500 iterations on original face images. 
       :align: center

       Basis images of LSNMF obtained after 500 iterations on original face images. The bases trained by LSNMF are additive
       but not spatially localized for representation of faces. Random VCol initialization algorithm is used. The number of
       subiterations for solving subproblems in LSNMF is a important issues. However, we stick to default and use 10 subiterations
       in this example. 


    .. figure:: /images/orl_faces_200_iters_small_NMF.png
       :scale: 70 %
       :alt: Basis images of NMF - Euclidean obtained after 200 iterations on reduced face images. 
       :align: center

       Basis images of NMF - Euclidean obtained after 200 iterations on reduced face images. The images show that
       the bases trained by NMF are additive but not spatially localized for representation of faces. The Euclidean
       distance of NMF estimate from target matrix is 33283.360. Random VCol initialization algorithm is used. 
       
       
    .. figure:: /images/orl_faces_200_iters_small_LSNMF.png
       :scale: 70 %
       :alt: Basis images of LSNMF obtained after 200 iterations on reduced face images.  
       :align: center

       Basis images of LSNMF obtained after 200 iterations on reduced face images. The bases trained by LSNMF are additive. The
       Euclidean distance of LSNMF estimate from target matrix is 29631.784 and projected gradient norm, which is used as 
       objective function in LSNMF is 7.9. Random VCol initialization algorithm is used. In LSNMF there is parameter beta, 
       we set is to 0.1. Beta is the rate of reducing the step size to satisfy the sufficient decrease condition. Smaller
       beta reduces the step size aggressively but may result in step size that is too small and the cost per iteration is thus
       higher. 
    
       
    .. figure:: /images/orl_faces_5_iters_small_PSMF_prior5.png
       :scale: 70 %
       :alt: Basis images of PSMF obtained after 5 iterations on reduced face images and with set prior parameter to 5.  
       :align: center

       Basis images of PSMF obtained after 5 iterations on reduced face images and with set prior parameter to 5. The
       bases trained from PSMF are both additive and spatially localized for representing faces. By setting prior to 5, in PSMF 
       the basis matrix is found under structural sparseness constraint that each row contains at most 5 non zero entries. This
       means, each row vector of target data matrix is explained by linear combination of at most 5 factors. Because we passed 
       prior as scalar and not list, uniform prior is taken, reflecting no prior knowledge on the distribution.  
       
       
    To run the example simply type::
        
        python orl_images.py
        
    or call the module's function::
    
        import nimfa.examples
        nimfa.examples.orl_images.run()
        
    .. note:: This example uses ``matplotlib`` library for producing visual interpretation of basis vectors. It uses PIL 
              library for displaying face images. 
"""

from os.path import dirname, abspath
from os.path import join
from warnings import warn

import numpy as np

import nimfa

try:
    from matplotlib.pyplot import savefig, imshow, set_cmap
except ImportError as exc:
    warn("Matplotlib must be installed to run ORL images example.")

try:
    from PIL.Image import open, fromarray, new
    from PIL.ImageOps import expand
except ImportError as exc:
    warn("PIL must be installed to run ORL images example.")


def run():
    """Run LSNMF on ORL faces data set."""
    # read face image data from ORL database
    V = read()
    # preprocess ORL faces data matrix
    V = preprocess(V)
    # run factorization
    W, _ = factorize(V)
    # plot parts-based representation
    plot(W)


def factorize(V):
    """
    Perform LSNMF factorization on the ORL faces data matrix. 
    
    Return basis and mixture matrices of the fitted factorization model. 
    
    :param V: The ORL faces data matrix. 
    :type V: `numpy.matrix`
    """
    lsnmf = nimfa.Lsnmf(V, seed="random_vcol", rank=25, max_iter=50, sub_iter=10,
                        inner_sub_iter=10, beta=0.1, min_residuals=1e-8)
    print("Algorithm: %s\nInitialization: %s\nRank: %d" % (lsnmf, lsnmf.seed, lsnmf.rank))
    fit = lsnmf()
    print("""Stats:
            - iterations: %d
            - final projected gradients norm: %5.3f
            - Euclidean distance: %5.3f""" % (fit.fit.n_iter, fit.distance(),
                                              fit.distance(metric='euclidean')))
    return fit.basis(), fit.coef()


def read():
    """
    Read face image data from the ORL database. The matrix's shape is 2576 (pixels) x 400 (faces). 
    
    Step through each subject and each image. Reduce the size of the images by a factor of 0.5. 
    
    Return the ORL faces data matrix. 
    """
    print("Reading ORL faces database")
    dir = join(dirname(dirname(abspath(__file__))), 'datasets', 'ORL_faces', 's')
    V = np.matrix(np.zeros((46 * 56, 400)))
    for subject in range(40):
        for image in range(10):
            im = open(join(dir + str(subject + 1), str(image + 1) + ".pgm"))
            # reduce the size of the image
            im = im.resize((46, 56))
            V[:, 10 * subject + image] = np.mat(np.asarray(im).flatten()).T
    return V


def preprocess(V):
    """
    Preprocess ORL faces data matrix as Stan Li, et. al.
    
    Return normalized and preprocessed data matrix. 
    
    :param V: The ORL faces data matrix. 
    :type V: `numpy.matrix`
    """
    print("Data preprocessing")
    min_val = V.min(axis=0)
    V = V - np.mat(np.ones((V.shape[0], 1))) * min_val
    max_val = V.max(axis=0) + 1e-4
    V = (255. * V) / (np.mat(np.ones((V.shape[0], 1))) * max_val) / 100.
    return V


def plot(W):
    """
    Plot basis vectors.
    
    :param W: Basis matrix of the fitted factorization model.
    :type W: `numpy.matrix`
    """
    set_cmap('gray')
    blank = new("L", (225 + 6, 280 + 6))
    for i in range(5):
        for j in range(5):
            basis = np.array(W[:, 5 * i + j])[:, 0].reshape((56, 46))
            basis = basis / np.max(basis) * 255
            basis = 255 - basis
            ima = fromarray(basis)
            expand(ima, border=1, fill='black')
            blank.paste(ima.copy(), (j * 46 + j, i * 56 + i))
    imshow(blank)
    savefig("orl_faces.png")


if __name__ == "__main__":
    """Run the ORL faces example."""
    run()




"""
    ############################
    Nmf_std (``models.nmf_std``)
    ############################
"""

from .nmf import *


class Nmf_std(Nmf):
    """
    Implementation of the standard model to manage factorizations that follow standard NMF model.
     
    It is the underlying model of matrix factorization and provides a general structure
    of standard NMF model.
    
    .. attribute:: W
        
        Basis matrix -- the first matrix factor in standard factorization
        
    .. attribute:: H
    
        Mixture matrix -- the second matrix factor in standard factorization
    """
    def __init__(self, params):
        """
        Construct factorization model that manages standard NMF models.
        
        :param params: MF runtime and algorithm parameters and options. For detailed
           explanation of the general model parameters see :mod:`mf_run`. For
           algorithm specific model options see documentation of chosen factorization method.
        :type params: `dict`
        """
        self.model_name = "std"
        self.V1 = None
        self.H1 = None
        Nmf.__init__(self, params)
        if sp.isspmatrix(self.V) and (self.V.data < 0).any() or not sp.isspmatrix(self.V) and (self.V < 0).any():
            raise utils.MFError("The input matrix contains negative elements.")

    def basis(self):
        """Return the matrix of basis vectors."""
        return self.W

    def target(self, idx=None):
        """
        Return the target matrix to estimate.
        
        :param idx: Used in the multiple NMF model. In standard NMF ``idx`` is always None.
        :type idx: None
        """
        return self.V

    def coef(self, idx=None):
        """
        Return the matrix of mixture coefficients.
        
        :param idx: Used in the multiple NMF model. In standard NMF ``idx`` is always None.
        :type idx: None
        """
        return self.H

    def fitted(self, idx=None):
        """
        Compute the estimated target matrix according to the NMF algorithm model.
        
        :param idx: Used in the multiple NMF model. In standard NMF ``idx`` is always None.
        :type idx: None
        """
        return dot(self.W, self.H)

    def distance(self, metric='euclidean', idx=None):
        """
        Return the loss function value.
        
        :param distance: Specify distance metric to be used. Possible are Euclidean and
           Kullback-Leibler (KL) divergence. Strictly, KL is not a metric.
        :type distance: `str` with values 'euclidean' or 'kl'

        :param idx: Used in the multiple NMF model. In standard NMF ``idx`` is always None.
        :type idx: None
        """
        if metric.lower() == 'euclidean':
            R = self.V - dot(self.W, self.H)
            return (power(R, 2)).sum()
        elif metric.lower() == 'kl':
            Va = dot(self.W, self.H)
            return (multiply(self.V, sop(elop(self.V, Va, div), op=np.log)) - self.V + Va).sum()
        else:
            raise utils.MFError("Unknown distance metric.")

    def residuals(self, idx=None):
        """
        Return residuals matrix between the target matrix and its NMF estimate.
        
        :param idx: Used in the multiple NMF model. In standard NMF ``idx`` is always None.
        :type idx: None
        """
        return self.V - dot(self.W, self.H)




"""
    This package contains nimfa library examples of usage. It demonstrates the following:
        
        #. Single run of the specified factorization algorithm.
        #. Multiple runs of the specified factorization algorithm.
        #. Tracking fitted results across multiple runs and tracking residuals error across single / multiple runs.
        #. Quality and performance measures of executed factorizations.
        #. Running factorization algorithms with algorithm specific settings and initializations. 
        
    Applications of factorization methods on both synthetic and real world data sets are provided. 
    
    Example using synthetic data set is intended as demonstration of the nimfa library since all currently implemented 
    factorization algorithms with different initialization methods and specific settings are ran. Others include 
    applications on real world data sets in:
    
        * bioinformatics,
        * functional genomics, 
        * text analysis,
        * image processing,
        * recommendation systems.
"""

from . import synthetic
from . import all_aml
from . import medulloblastoma
from . import cbcl_images
from . import documents
from . import orl_images
from . import recommendations
from . import gene_func_prediction




"""
#############################################
Random_vcol (``methods.seeding.random_vcol``)
#############################################

Random Vcol [Albright2006]_ is inexpensive initialization method for nonnegative
matrix factorization. Random Vcol forms an initialization of each column of the
basis matrix (W) by averaging p random columns of target matrix (V). Similarly,
Random Vcol forms an initialization of each row of the mixture matrix (H) by
averaging p random rows of target matrix (V). It makes more sense to build the
basis vectors from the given data than to form completely random basis vectors,
as random initialization does. Sparse matrices are built from the original
sparse data.

Method's performance lies between random initialization and centroid
initialization, which is built from the centroid decomposition.
"""

from nimfa.utils.linalg import *

__all__ = ['Random_vcol']


class Random_vcol(object):

    def __init__(self):
        self.name = "random_vcol"

    def initialize(self, V, rank, options):
        """
        Return initialized basis and mixture matrix. Initialized matrices are of
        the same type as passed target matrix.
        
        :param V: Target matrix, the matrix for MF method to estimate. 
        :type V: One of the :class:`scipy.sparse` sparse matrices types or or
                :class:`numpy.matrix`
        :param rank: Factorization rank. 
        :type rank: `int`
        :param options: Specify the algorithm and model specific options (e.g. initialization of
                extra matrix factor, seeding parameters).
                    
                Option ``p_c`` represents the number of columns of target matrix used
                to average the column of basis matrix. Default value for ``p_c`` is
                1/5 * (target.shape[1]).

                Option ``p_r`` represent the number of rows of target matrix used to
                average the row of basis matrix. Default value for ``p_r`` is 1/5 * (target.shape[0]).
        :type options: `dict`
        """
        self.rank = rank
        self.p_c = options.get('p_c', int(ceil(1. / 5 * V.shape[1])))
        self.p_r = options.get('p_r', int(ceil(1. / 5 * V.shape[0])))
        self.prng = np.random.RandomState()
        if sp.isspmatrix(V):
            self.W = sp.lil_matrix((V.shape[0], self.rank))
            self.H = sp.lil_matrix((self.rank, V.shape[1]))
        else:
            self.W = np.mat(np.zeros((V.shape[0], self.rank)))
            self.H = np.mat(np.zeros((self.rank, V.shape[1])))
        for i in range(self.rank):
            self.W[:, i] = V[:, self.prng.randint(
                low=0, high=V.shape[1], size=self.p_c)].mean(axis=1)
            self.H[i, :] = V[
                self.prng.randint(low=0, high=V.shape[0], size=self.p_r), :].mean(axis=0)
        # return sparse or dense initialization
        if sp.isspmatrix(V):
            return self.W.tocsr(), self.H.tocsr()
        else:
            return self.W, self.H

    def __repr__(self):
        return "random_vcol.Random_vcol()"

    def __str__(self):
        return self.name



import nimfa

import numpy as np

V = np.array([[1, 2, 3], [4, 5, 6], [6, 7, 8]])
print('Target:\n%s' % V)

# Initialization callback function
def init_info(model):
    print('Initialized basis matrix:\n%s' % model.basis())
    print('Initialized  mixture matrix:\n%s' % model.coef())

# Callback is called after initialization and prior to factorization in each run
icm = nimfa.Icm(V, seed='random_c', max_iter=10, rank=3, callback_init=init_info)
icm_fit = icm()

W = icm_fit.basis()
print('Basis matrix:\n%s' % W)
print(W)

H = icm_fit.coef()
print('Mixture matrix:\n%s' % H)

sm = icm_fit.summary()
print('Rss: %5.3f' % sm['rss'])
print('Evar: %5.3f' % sm['evar'])
print('Iterations: %d' % sm['n_iter'])
print('KL divergence: %5.3f' % sm['kl'])
print('Euclidean distance: %5.3f' % sm['euclidean'])



"""
    #####################
    Smf (``models.smf``)
    #####################
"""

import nimfa.utils.utils as utils
from nimfa.utils.linalg import *
from nimfa.methods import seeding


class Smf(object):
    """
    This class defines a common interface / model to handle standard MF models in
    a generic way.
    
    It contains definitions of the minimum set of generic methods that are used in 
    common computations and matrix factorizations. Besides it contains some quality
    and performance measures about factorizations.
    """
    def __init__(self, params):
        self.model_name = "smf"
        self.__dict__.update(params)
        self.V1 = None
        self.H1 = None
        # do not copy target and factor matrices into the program
        if sp.isspmatrix(self.V):
            self.V = self.V.tocsr().astype('d')
        else:
            self.V = np.asmatrix(self.V) if self.V.dtype == np.dtype(
                float) else np.asmatrix(self.V, dtype='d')
        if self.W is not None or self.H is not None or self.H1 is not None:
            raise utils.MFError("Fixed initialized is not supported by SMF model.")
        self._compatibility()

    def __call__(self):
        """Run the specified MF algorithm."""
        return self.factorize()

    def basis(self):
        """Return the matrix of basis vectors (factor 1 matrix)."""
        return self.W

    def target(self, idx=None):
        """
        Return the target matrix to estimate.
        
        :param idx: Used in the multiple MF model. In standard MF ``idx`` is always None.
        :type idx: None
        """
        return self.V

    def coef(self, idx=None):
        """
        Return the matrix of mixture coefficients (factor 2 matrix).
        
        :param idx: Used in the multiple MF model. In standard MF ``idx`` is always None.
        :type idx: None
        """
        return self.H

    def fitted(self, idx=None):
        """
        Compute the estimated target matrix according to the MF algorithm model.
        
        :param idx: Used in the multiple MF model. In standard MF ``idx`` is always None.
        :type idx: None
        """
        return dot(self.W, self.H)

    def distance(self, metric='euclidean', idx=None):
        """
        Return the loss function value.
        
        :param distance: Specify distance metric to be used. Possible are Euclidean and
           Kullback-Leibler (KL) divergence. Strictly, KL is not a metric.
        :type distance: `str` with values 'euclidean' or 'kl'

        :param idx: Used in the multiple MF model. In standard MF ``idx`` is always None.
        :type idx: None
        """
        if metric.lower() == 'euclidean':
            R = self.V - dot(self.W, self.H)
            return power(R, 2).sum()
        elif metric.lower() == 'kl':
            Va = dot(self.W, self.H)
            return (multiply(self.V, sop(elop(self.V, Va, div), op=log)) - self.V + Va).sum()
        else:
            raise utils.MFError("Unknown distance metric.")

    def residuals(self, idx=None):
        """
        Return residuals matrix between the target matrix and its MF estimate.
        
        :param idx: Used in the multiple MF model. In standard MF ``idx`` is always None.
        :type idx: None
        """
        return self.V - dot(self.W, self.H)

    def _compatibility(self):
        """
        Check if chosen seeding method is compatible with chosen factorization
        method or fixed initialization is passed.

        :param mf_model: The underlying initialized model of matrix factorization.
        :type mf_model: Class inheriting :class:`models.nmf.Nmf`
        """
        W = self.basis()
        H = self.coef(0)
        H1 = self.coef(1) if self.model_name == 'mm' else None
        if self.seed is None and W is None and H is None and H1 is None:
            self.seed = None if "none" in self.aseeds else "random"
        if W is not None and H is not None:
            if self.seed is not None and self.seed is not "fixed":
                raise utils.MFError("Initial factorization is fixed.")
            else:
                self.seed = seeding.fixed.Fixed()
                self.seed._set_fixed(W=W, H=H, H1=H1)
        self.__is_smdefined()
        self.__compatibility()

    def __is_smdefined(self):
        """Check if MF and seeding methods are well defined."""
        if isinstance(self.seed, str):
            if self.seed in seeding.methods:
                self.seed = seeding.methods[self.seed]()
            else:
                raise utils.MFError("Unrecognized seeding method.")
        else:
            if not str(self.seed).lower() in seeding.methods:
                raise utils.MFError("Unrecognized seeding method.")

    def __compatibility(self):
        """Check if MF model is compatible with the seeding method."""
        if not str(self.seed).lower() in self.aseeds:
            raise utils.MFError("MF model is incompatible with the seeding method.")




"""
#########################
Utils (``utils.utils``)
#########################
"""

class MFError(Exception):

    """
    Generic Python exception derived object raised by nimfa library. 
    
    This is general purpose exception class, derived from Python common base
    class for all non-exit exceptions. It is programmatically raised in nimfa
    library functions when:
        
        #. linear algebra related condition prevents further correct execution
        of a function;
        #. user input parameters are ambiguous or not in correct format.
    """

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return repr(self.msg)



import numpy as np

import nimfa

V = np.random.rand(40, 100)
pmfcc = nimfa.Pmfcc(V, seed="random_vcol", rank=10, max_iter=30,
                    theta=np.random.rand(V.shape[1], V.shape[1]))
pmfcc_fit = pmfcc()



import numpy as np

import nimfa

V = np.array([[1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12], [13, 14, 15, 16]])
print('Target:\n%s' % V)

lsnmf = nimfa.Lsnmf(V, seed='random_vcol', max_iter=10, rank=3, track_error=True)
lsnmf_fit = lsnmf()

W = lsnmf_fit.basis()
print('Basis matrix:\n%s' % W)

H = lsnmf_fit.coef()
print('Mixture matrix:\n%s' % H)

r = lsnmf.estimate_rank(rank_range=[2,3,4], what=['rss'])
pp_r = '\n'.join('%d: %5.3f' % (rank, vals['rss']) for rank, vals in r.items())
print('Rank estimate:\n%s' % pp_r)



import numpy as np

import nimfa

V = np.random.rand(40, 100)
lfnmf = nimfa.Lfnmf(V, seed=None, W=np.random.rand(V.shape[0], 10),
                    H=np.random.rand(10, V.shape[1]), rank=10, max_iter=12,
                    alpha=0.01)
lfnmf_fit = lfnmf()



import numpy as np

import nimfa

V = np.array([[1, 2, 3], [4, 5, 6], [6, 7, 8]])
print('Target:\n%s' % V)

lsnmf = nimfa.Lsnmf(V, seed='random_vcol', max_iter=10, rank=3, track_error=True)
lsnmf_fit = lsnmf()

W = lsnmf_fit.basis()
print('Basis matrix:\n%s' % W)

H = lsnmf_fit.coef()
print('Mixture matrix:\n%s' % H)

# Objective function value for each iteration
print('Error tracking:\n%s' % lsnmf_fit.fit.tracker.get_error())

sm = lsnmf_fit.summary()
print('Rss: %5.3f' % sm['rss'])
print('Evar: %5.3f' % sm['evar'])
print('Iterations: %d' % sm['n_iter'])




"""
    ##############################
    Mf_track (``models.mf_track``)
    ##############################
"""


class Mf_track():
    """
    Base class for tracking MF fitted model across multiple runs of the factorizations or tracking
    the residuals error across iterations of single/multiple runs. 
    
    The purpose of this class is to store matrix factors from multiple runs of the algorithm which can then be used
    for performing quality and performance measures. Because of additional space consumption for storing multiple 
    matrix factors, tracking is used only if explicitly specified by user through runtime option. In summary, when
    tracking factors, the following is retained from each run:
        
        #. fitted factorization model depending on the factorization method; 
        #. final value of objective function; 
        #. performed number of iterations. 
        
    Instead of tracking fitted factorization model, callback function can be set, which will be called after each 
    factorization run. For more details see :mod:`mf_run`.
    
    The purpose of this class is to store residuals across iterations which can then be used for plotting the trajectory 
    of the residuals track or estimating proper number of iterations. 
    """
    def __init__(self):
        """
        Construct model for tracking fitted factorization model across multiple runs or tracking the residuals error across iterations. 
        """
        self._factors = {}
        self._residuals = {}

    def track_error(self, run, residuals):
        """
        Add residuals error after one iteration. 
        
        :param run: Specify the run to which ``residuals`` belongs. Error tracking can be
           also used if multiple runs are enabled.
        :type run: `int`

        :param residuals: Residuals between the target matrix and its MF estimate.
        :type residuals: `float`
        """
        self._residuals.setdefault(run, [])
        self._residuals[run].append(residuals)

    def track_factor(self, run, **track_model):
        """
        Add matrix factorization factors (and method specific model data) after one factorization run.
        
        :param run: Specify the run to which ``track_model`` belongs.
        :type run: 'int'
        :param track_model: Matrix factorization factors.
        :type track_model:  algorithm specific
        """
        self._factors[run] = t_model(track_model)

    def get_factor(self, run=0):
        """
        Return matrix factorization factors from run :param:`run`.
        
        :param run: Saved factorization factors (and method specific model data) of
           ``run``'th run are returned.
        :type run: `int`
        """
        return self._factors[run]

    def get_error(self, run=0):
        """
        Return residuals track from one run of the factorization.
        
        :param run: Specify the run of which error track is desired. By default ``run`` is 1.
        :type run: `int`
        """
        return self._residuals[run]


class t_model:

    """
    Tracking factors model.
    """

    def __init__(self, td):
        self.__dict__.update(td)




# THIS FILE IS GENERATED FROM NIMFA SETUP.PY
short_version = '1.4.0'
version = '1.4.0'
full_version = '1.4.0'
git_revision = 'eecbb05c4506ef482e788c8e6972c386dcbdb34a'
release = True

if not release:
    version = full_version
    short_version += ".dev"



import numpy as np

import nimfa

V = np.array([[1, 2, 3], [4, 5, 6], [6, 7, 8]])
print('Target:\n%s' % V)

lsnmf = nimfa.Lsnmf(V, max_iter=10, rank=3)
lsnmf_fit = lsnmf()

W = lsnmf_fit.basis()
print('Basis matrix:\n%s' % W)

H = lsnmf_fit.coef()
print('Mixture matrix:\n%s' % H)

print('K-L divergence: %5.3f' % lsnmf_fit.distance(metric='kl'))

print('Rss: %5.3f' % lsnmf_fit.fit.rss())
print('Evar: %5.3f' % lsnmf_fit.fit.evar())
print('Iterations: %d' % lsnmf_fit.n_iter)
print('Target estimate:\n%s' % np.dot(W, H))



import numpy as np

import nimfa

V = np.random.rand(40, 100)
icm = nimfa.Icm(V, seed="nndsvd", rank=10, max_iter=12, iiter=20,
                alpha=np.random.randn(V.shape[0], 10), beta=np.random.randn(10, V.shape[1]),
                theta=0., k=0., sigma=1.)
icm_fit = icm()



import os
from glob import glob
from setuptools import setup, find_packages
import subprocess
import imp


DISTNAME = 'nimfa'
MAINTAINER = 'Marinka Zitnik'
MAINTAINER_EMAIL = 'marinka@cs.stanford.edu'
DESCRIPTION = 'A Python module for nonnegative matrix factorization'
LONG_DESCRIPTION = open('README.md').read()
URL = 'http://nimfa.biolab.si'
DOWNLOAD_URL = 'http://github.com/marinkaz/nimfa'
KEYWORDS = ['matrix factorization', 'nonnegative matrix factorization',
            'bioinformatics', 'data mining', 'machine learning']
LICENSE = 'BSD'
VERSION = '1.4.0'
ISRELEASED = True

INSTALL_REQUIRES = (
    'numpy>=1.7.0',
    'scipy>=0.12.0',
)


def get_package_data(topdir, excluded=set()):
    retval = []
    for dirname, subdirs, files in os.walk(os.path.join(DISTNAME, topdir)):
        for x in excluded:
            if x in subdirs:
                subdirs.remove(x)
        retval.append(os.path.join(dirname[len(DISTNAME)+1:], '*.*'))
    return retval


def get_data_files(dest, source):
    retval = []
    for dirname, subdirs, files in os.walk(source):
        retval.append(
            (os.path.join(dest, dirname[len(source)+1:]), glob(os.pathjoin(dirname, '*.*')))
        )
    return retval


# Return the git revision as a string
def git_version():
    """Return the git revision as a string.

    Copied from numpy setup.py
    """
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(cmd, stdout = subprocess.PIPE, env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', 'HEAD'])
        GIT_REVISION = out.strip().decode('ascii')
    except OSError:
        GIT_REVISION = "Unknown"
    return GIT_REVISION


def write_version_py(filename='nimfa/version.py'):
    # Copied from numpy setup.py
    cnt = """
# THIS FILE IS GENERATED FROM NIMFA SETUP.PY
short_version = '%(version)s'
version = '%(version)s'
full_version = '%(full_version)s'
git_revision = '%(git_revision)s'
release = %(isrelease)s

if not release:
    version = full_version
    short_version += ".dev"
"""
    FULLVERSION = VERSION
    if os.path.exists('.git'):
        GIT_REVISION = git_version()
    elif os.path.exists('nimfa/version.py'):
        # must be a source distribution, use existing version file
        version = imp.load_source('nimfa.version', 'nimfa/version.py')
        GIT_REVISION = version.git_revision
    else:
        GIT_REVISION = 'Unknown'

    if not ISRELEASED:
        FULLVERSION += '.dev0+' + GIT_REVISION[:7]

    a = open(filename, 'w')
    try:
        a.write(cnt % {'version': VERSION,
                       'full_version': FULLVERSION,
                       'git_revision': GIT_REVISION,
                       'isrelease': str(ISRELEASED)})
    finally:
        a.close()



def setup_package():
    write_version_py()
    setup(
        name=DISTNAME,
        version=VERSION,
        author=MAINTAINER,
        author_email=MAINTAINER_EMAIL,
        maintainer=MAINTAINER,
        maintainer_email=MAINTAINER_EMAIL,
        description=DESCRIPTION,
        url=URL,
        download_url=DOWNLOAD_URL,
        keywords=KEYWORDS,
        install_requires=INSTALL_REQUIRES,
        packages=find_packages(),
        package_dir={DISTNAME: './nimfa'},
        package_data={DISTNAME: get_package_data('datasets')},
        license=LICENSE,
        long_description_content_type='text/markdown',
        long_description=LONG_DESCRIPTION,
        classifiers=['Intended Audience :: Science/Research',
                     'Intended Audience :: Developers',
                     'License :: OSI Approved',
                     'Programming Language :: Python',
                     'Topic :: Software Development',
                     'Topic :: Scientific/Engineering',
                     'Topic :: Scientific/Engineering :: Artificial Intelligence',
                     'Topic :: Scientific/Engineering :: Bio-Informatics',
                     'Programming Language :: Python :: 2',
                     'Programming Language :: Python :: 3',],
        )


if __name__ == "__main__":
    setup_package()




"""
#######################################
Lsnmf (``methods.factorization.lsnmf``)
#######################################

**Alternating Nonnegative Least Squares Matrix Factorization Using Projected
Gradient (bound constrained optimization) method for each subproblem
(LSNMF)** [Lin2007]_.

It converges faster than the popular multiplicative update approach. 

Algorithm relies on efficiently solving bound constrained subproblems. They are
solved using the projected gradient method. Each subproblem contains some (m)
independent nonnegative least squares problems. Not solving these separately but
treating them together is better because of: problems are closely related,
sharing the same constant matrices; all operations are matrix based, which
saves computational time.

The main task per iteration of the subproblem is to find a step size alpha such
that a sufficient decrease condition of bound constrained problem is satisfied.
In alternating least squares, each subproblem involves an optimization procedure
and requires a stopping condition. A common way to check whether current solution
is close to a stationary point is the form of the projected gradient [Lin2007]_.

.. literalinclude:: /code/snippet_lsnmf.py

"""

from nimfa.models import *
from nimfa.utils import *
from nimfa.utils.linalg import *

__all__ = ['Lsnmf']


class Lsnmf(nmf_std.Nmf_std):
    """
    :param V: The target matrix to estimate.
    :type V: Instance of the :class:`scipy.sparse` sparse matrices types,
       :class:`numpy.ndarray`, :class:`numpy.matrix` or tuple of instances of
       the latter classes.

    :param seed: Specify method to seed the computation of a factorization. If
       specified :param:`W` and :param:`H` seeding must be None. If neither seeding
       method or initial fixed factorization is specified, random initialization is
       used.
    :type seed: `str` naming the method or :class:`methods.seeding.nndsvd.Nndsvd`
       or None

    :param W: Specify initial factorization of basis matrix W. Default is None.
       When specified, :param:`seed` must be None.
    :type W: :class:`scipy.sparse` or :class:`numpy.ndarray` or
       :class:`numpy.matrix` or None

    :param H: Specify initial factorization of mixture matrix H. Default is None.
       When specified, :param:`seed` must be None.
    :type H: Instance of the :class:`scipy.sparse` sparse matrices types,
       :class:`numpy.ndarray`, :class:`numpy.matrix`, tuple of instances of the
       latter classes or None

    :param rank: The factorization rank to achieve. Default is 30.
    :type rank: `int`

    :param n_run: It specifies the number of runs of the algorithm. Default is
       1. If multiple runs are performed, fitted factorization model with the
       lowest objective function value is retained.
    :type n_run: `int`

    :param callback: Pass a callback function that is called after each run when
       performing multiple runs. This is useful if one wants to save summary
       measures or process the result before it gets discarded. The callback
       function is called with only one argument :class:`models.mf_fit.Mf_fit` that
       contains the fitted model. Default is None.
    :type callback: `function`

    :param callback_init: Pass a callback function that is called after each
       initialization of the matrix factors. In case of multiple runs the function
       is called before each run (more precisely after initialization and before
       the factorization of each run). In case of single run, the passed callback
       function is called after the only initialization of the matrix factors.
       This is useful if one wants to obtain the initialized matrix factors for
       further analysis or additional info about initialized factorization model.
       The callback function is called with only one argument
       :class:`models.mf_fit.Mf_fit` that (among others) contains also initialized
       matrix factors. Default is None.
    :type callback_init: `function`

    :param track_factor: When :param:`track_factor` is specified, the fitted
        factorization model is tracked during multiple runs of the algorithm. This
        option is taken into account only when multiple runs are executed
        (:param:`n_run` > 1). From each run of the factorization all matrix factors
        are retained, which can be very space consuming. If space is the problem
        setting the callback function with :param:`callback` is advised which is
        executed after each run. Tracking is useful for performing some quality or
        performance measures (e.g. cophenetic correlation, consensus matrix,
        dispersion). By default fitted model is not tracked.
    :type track_factor: `bool`

    :param track_error: Tracking the residuals error. Only the residuals from
        each iteration of the factorization are retained. Error tracking is not
        space consuming. By default residuals are not tracked and only the final
        residuals are saved. It can be used for plotting the trajectory of the
        residuals.
    :type track_error: `bool`

    :param sub_iter: Maximum number of subproblem iterations. Default value is 10.
    :type sub_iter: `int`

    :param inner_sub_iter: Number of inner iterations when solving subproblems.
       Default value is 10.
    :type inner_sub_iter: `int`

    :param beta: The rate of reducing the step size to satisfy the sufficient
       decrease condition when solving subproblems. Smaller beta more aggressively
       reduces the step size, but may cause the step size being too small. Default
       value is 0.1.
    :type beta: `float`

    **Stopping criterion**

    Factorization terminates if any of specified criteria is satisfied.

    :param max_iter: Maximum number of factorization iterations. Note that the
       number of iterations depends on the speed of method convergence. Default
       is 30.
    :type max_iter: `int`

    :param min_residuals: Minimal required improvement of the residuals from the
       previous iteration. They are computed between the target matrix and its MF
       estimate using the objective function associated to the MF algorithm.
       Default is None.
    :type min_residuals: `float`

    :param test_conv: It indicates how often convergence test is done. By
       default convergence is tested each iteration.
    :type test_conv: `int`
    """
    def __init__(self, V, seed=None, W=None, H=None, H1=None,
                 rank=30, max_iter=30, min_residuals=1e-5, test_conv=None,
                 n_run=1, callback=None, callback_init=None, track_factor=False,
                 track_error=False, sub_iter=10, inner_sub_iter=10, beta=0.1, **options):
        self.name = "lsnmf"
        self.aseeds = ["random", "fixed", "nndsvd", "random_c", "random_vcol"]
        nmf_std.Nmf_std.__init__(self, vars())
        self.min_residuals = 1e-5 if not self.min_residuals else self.min_residuals
        self.tracker = mf_track.Mf_track() if self.track_factor and self.n_run > 1 \
                                              or self.track_error else None

    def factorize(self):
        """
        Compute matrix factorization.
         
        Return fitted factorization model.
        """
        for run in range(self.n_run):
            self.W, self.H = self.seed.initialize(
                self.V, self.rank, self.options)
            self.gW = dot(self.W, dot(self.H, self.H.T)) - dot(
                self.V, self.H.T)
            self.gH = dot(dot(self.W.T, self.W), self.H) - dot(
                self.W.T, self.V)
            self.init_grad = norm(vstack(self.gW, self.gH.T), p='fro')
            self.epsW = max(1e-3, self.min_residuals) * self.init_grad
            self.epsH = self.epsW
            # iterW and iterH are not parameters, as these values are used only
            # in first objective computation
            self.iterW = 10
            self.iterH = 10
            c_obj = sys.float_info.max
            best_obj = c_obj if run == 0 else best_obj
            iter = 0
            if self.callback_init:
                self.final_obj = c_obj
                self.n_iter = iter
                mffit = mf_fit.Mf_fit(self)
                self.callback_init(mffit)
            while self.is_satisfied(c_obj, iter):
                self.update()
                iter += 1
                c_obj = self.objective(
                ) if not self.test_conv or iter % self.test_conv == 0 else c_obj
                if self.track_error:
                    self.tracker.track_error(run, c_obj)
            if self.callback:
                self.final_obj = c_obj
                self.n_iter = iter
                mffit = mf_fit.Mf_fit(self)
                self.callback(mffit)
            if self.track_factor:
                self.tracker.track_factor(
                    run, W=self.W, H=self.H, final_obj=c_obj, n_iter=iter)
            # if multiple runs are performed, fitted factorization model with
            # the lowest objective function value is retained
            if c_obj <= best_obj or run == 0:
                best_obj = c_obj
                self.n_iter = iter
                self.final_obj = c_obj
                mffit = mf_fit.Mf_fit(copy.deepcopy(self))

        mffit.fit.tracker = self.tracker
        return mffit

    def is_satisfied(self, c_obj, iter):
        """
        Compute the satisfiability of the stopping criteria based on stopping
        parameters and objective function value.
        
        Return logical value denoting factorization continuation. 
        
        :param c_obj: Current objective function value. 
        :type c_obj: `float`
        :param iter: Current iteration number. 
        :type iter: `int`
        """
        if self.max_iter and self.max_iter <= iter:
            return False
        if self.test_conv and iter % self.test_conv != 0:
            return True
        if iter > 0 and c_obj < self.min_residuals * self.init_grad:
            return False
        if self.iterW == 0 and self.iterH == 0 and self.epsW + self.epsH < self.min_residuals * self.init_grad:
            # There was no move in this iteration
            return False
        return True

    def update(self):
        """Update basis and mixture matrix."""
        self.W, self.gW, self.iterW = self._subproblem(
            self.V.T, self.H.T, self.W.T, self.epsW)
        self.W = self.W.T
        self.gW = self.gW.T
        self.epsW = 0.1 * self.epsW if self.iterW == 0 else self.epsW
        self.H, self.gH, self.iterH = self._subproblem(
            self.V, self.W, self.H, self.epsH)
        self.epsH = 0.1 * self.epsH if self.iterH == 0 else self.epsH

    def _subproblem(self, V, W, Hinit, epsH):
        """
        Optimization procedure for solving subproblem (bound-constrained
        optimization).
        
        Return output solution, gradient and number of used iterations.
        
        :param V: Constant matrix.
        :type V: :class:`scipy.sparse` of format csr, csc, coo, bsr, dok, lil,
           dia or :class:`numpy.matrix`

        :param W: Constant matrix.
        :type W: :class:`scipy.sparse` of format csr, csc, coo, bsr, dok, lil,
           dia or :class:`numpy.matrix`

        :param Hinit: Initial solution to the subproblem.
        :type Hinit: :class:`scipy.sparse` of format csr, csc, coo, bsr, dok,
           lil, dia or :class:`numpy.matrix`

        :param epsH: Tolerance for termination.
        :type epsH: `float`
        """
        H = Hinit
        WtV = dot(W.T, V)
        WtW = dot(W.T, W)
        # alpha is step size regulated by beta
        # beta is the rate of reducing the step size to satisfy the sufficient
        # decrease condition smaller beta more aggressively reduces the step
        # size, but may cause the step size alpha being too small
        alpha = 1.
        for iter in range(self.sub_iter):
            grad = dot(WtW, H) - WtV
            projgrad = norm(self.__extract(grad, H))
            if projgrad < epsH:
                break
            # search for step size alpha
            for n_iter in range(self.inner_sub_iter):
                Hn = max(H - alpha * grad, 0)
                d = Hn - H
                gradd = multiply(grad, d).sum()
                dQd = multiply(dot(WtW, d), d).sum()
                suff_decr = 0.99 * gradd + 0.5 * dQd < 0
                if n_iter == 0:
                    decr_alpha = not suff_decr
                    Hp = H
                if decr_alpha:
                    if suff_decr:
                        H = Hn
                        break
                    else:
                        alpha *= self.beta
                else:
                    if not suff_decr or self.__alleq(Hp, Hn):
                        H = Hp
                        break
                    else:
                        alpha /= self.beta
                        Hp = Hn
        return H, grad, iter

    def objective(self):
        """Compute projected gradients norm."""
        return norm(vstack([self.__extract(self.gW, self.W), self.__extract(self.gH, self.H)]))

    def __alleq(self, X, Y):
        """
        Check element wise comparison for dense, sparse, mixed matrices.
        
        :param X: First input matrix.
        :type X: :class:`scipy.sparse` of format csr, csc, coo, bsr, dok, lil,
           dia or :class:`numpy.matrix`

        :param Y: Second input matrix.
        :type Y: :class:`scipy.sparse` of format csr, csc, coo, bsr, dok, lil,
           dia or :class:`numpy.matrix`
        """
        if sp.isspmatrix(X) and sp.isspmatrix(Y):
            X = X.tocsr()
            Y = Y.tocsr()
            if not np.all(X.data == Y.data):
                return False
            r1, c1 = X.nonzero()
            r2, c2 = Y.nonzero()
            if not np.all(r1 == r2) or not np.all(c1 == c2):
                return False
            else:
                return True
        else:
            return np.all(X == Y)

    def __extract(self, X, Y):
        """
        Extract elements for projected gradient norm.
        
        :param X: Gradient matrix.
        :type X: :class:`scipy.sparse` of format csr, csc, coo, bsr, dok,
           lil, dia or :class:`numpy.matrix`

        :param Y: Input matrix. 
        :type Y: :class:`scipy.sparse` of format csr, csc, coo, bsr, dok, lil,
           dia or :class:`numpy.matrix`
        """
        if sp.isspmatrix(X):
            X = X.tocsr()
            r1, c1 = X.nonzero()
            if r1.size != 0:
                xt = X[r1, c1] < 0
                xt = np.array(xt)
                xt = xt[0, :] if xt.shape[0] == 1 else xt[:, 0]
                r1 = r1[xt]
                c1 = c1[xt]

            Y = Y.tocsr()
            r2, c2 = Y.nonzero()
            if r2.size != 0:
                yt = Y[r2, c2] > 0
                yt = np.array(yt)
                yt = yt[0, :] if yt.shape[0] == 1 else yt[:, 0]
                r2 = r2[yt]
                c2 = c2[yt]

            idx1 = list(zip(r1, c1))
            idx2 = list(zip(r2, c2))

            idxf = set(idx1).union(set(idx2))
            rf, cf = list(zip(*idxf))
            return X[rf, cf].T
        else:
            return X[np.logical_or(X < 0, Y > 0)].flatten().T

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name
